
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

      SUBROUTINE HRPRODLOSS

C***********************************************************************
C
C  FUNCTION: To compute the production and loss rates for EBI species
C
C  PRECONDITIONS: For the CRACMM1_AQ mechanism
C
C  RETURN VALUES: None
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, Mar 12, 2022
C
C   18 Jul 14 B.Hutzell: revised to use real(8) variables

C***********************************************************************
      USE HRDATA

      IMPLICIT NONE


C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
!     CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


C.....LOCAL VARIABLES:


C***********************************************************************


      PROD(  H2O2        ) =                 RXRAT(    49 )      ! HO2+HO2=H2O2
     &                     +                 RXRAT(    50 )      ! HO2+HO2=H2O2
     &                     +    9.0000D-02 * RXRAT(    87 )      ! ISO+O3=0.9000D-01*H2O2+...
     &                     +    6.8000D-02 * RXRAT(   143 )      ! OLT+O3=0.6800D-01*H2O2+...
     &                     +    2.6000D-02 * RXRAT(   144 )      ! OLI+O3=0.2600D-01*H2O2+...
     &                     +    5.0000D-02 * RXRAT(   145 )      ! API+O3=0.5000D-01*H2O2+...
     &                     +    5.0000D-02 * RXRAT(   146 )      ! LIM+O3=0.5000D-01*H2O2+...
      LOSS(  H2O2        ) =                 RXRAT(     3 )      ! H2O2=0.2000D+01*HO
     &                     +                 RXRAT(    51 )      ! H2O2+HO=HO2

      PROD(  HNO3        ) =                 RXRAT(    55 )      ! NO+HO2=HNO3
     &                     +                 RXRAT(    60 )      ! NO2+HO=HNO3
     &                     +    3.0000D-01 * RXRAT(    63 )      ! NO3+HO2=0.3000D+00*HNO3+...
     &                     +    2.0000D+00 * RXRAT(    69 )      ! N2O5=0.2000D+01*HNO3
     &                     +                 RXRAT(   162 )      ! HCHO+NO3=HNO3+CO+HO2
     &                     +                 RXRAT(   163 )      ! ACD+NO3=HNO3+ACO3
     &                     +                 RXRAT(   164 )      ! ALD+NO3=HNO3+RCO3
     &                     +    3.2000D-01 * RXRAT(   165 )      ! MACR+NO3=0.3200D+00*HNO3+...
     &                     +                 RXRAT(   167 )      ! GLY+NO3=HNO3+0.2000D+01*CO+HO2
     &                     +                 RXRAT(   168 )      ! MGLY+NO3=HNO3+CO+ACO3
     &                     +    4.2400D-01 * RXRAT(   169 )      ! PHEN+NO3=0.4240D+00*HNO3+...
     &                     +    4.0000D-01 * RXRAT(   170 )      ! CSL+NO3=0.4000D+00*HNO3+...
     &                     +                 RXRAT(   171 )      ! MCT+NO3=HNO3+MCTO
     &                     +    3.2000D-01 * RXRAT(   392 )      ! ACRO+NO3=0.3200D+00*HNO3+...
     &                     +    2.0000D+00 * RXRAT(   422 )      ! N2O5=0.2000D+01*HNO3
     &                     +    5.0000D-01 * RXRAT(   423 )      ! NO2=0.5000D+00*HNO3+...
      LOSS(  HNO3        ) =                 RXRAT(     8 )      ! HNO3=HO+NO2
     &                     +                 RXRAT(    61 )      ! HNO3+HO=NO3

      PROD(  HCHO        ) =    3.5000D-01 * RXRAT(    16 )      ! UALD=0.3500D+00*HCHO+...
     &                     +                 RXRAT(    21 )      ! HKET=HCHO+ACO3+HO2
     &                     +    6.7000D-01 * RXRAT(    22 )      ! MACR=0.6700D+00*HCHO+...
     &                     +                 RXRAT(    25 )      ! GLY=HCHO+CO
     &                     +                 RXRAT(    31 )      ! OP1=HCHO+HO2+HO
     &                     +    8.8000D-01 * RXRAT(    87 )      ! ISO+O3=0.8800D+00*HCHO+...
     &                     +    3.0000D-01 * RXRAT(    88 )      ! ISO+NO3=0.3000D+00*HCHO+...
     &                     +    3.0000D-01 * RXRAT(    91 )      ! ISOP+NO=0.3000D+00*HCHO+...
     &                     +    9.0000D-01 * RXRAT(    94 )      ! INO2+NO=0.9000D+00*HCHO+...
     &                     +    4.0000D-01 * RXRAT(    95 )      ! IPN+HO2=0.4000D+00*HCHO+...
     &                     +    6.0000D-01 * RXRAT(    97 )      ! IHN+NO=0.6000D+00*HCHO+...
     &                     +    6.0000D-01 * RXRAT(    98 )      ! ISHP+HO=0.6000D+00*HCHO+...
     &                     +    6.0000D-01 * RXRAT(    99 )      ! ISHP=0.6000D+00*HCHO+...
     &                     +                 RXRAT(   126 )      ! MOH+HO=HCHO+HO2
     &                     +    3.5000D-01 * RXRAT(   130 )      ! OP1+HO=0.3500D+00*HCHO+...
     &                     +    3.5000D-01 * RXRAT(   136 )      ! PAA+HO=0.3500D+00*HCHO+...
     &                     +                 RXRAT(   137 )      ! PAN+HO=HCHO+NO3+XO2
     &                     +                 RXRAT(   138 )      ! PPN+HO=HCHO+NO3+XO2
     &                     +                 RXRAT(   142 )      ! ETE+O3=HCHO+0.1500D+00*HO2+...
     &                     +    5.6000D-01 * RXRAT(   143 )      ! OLT+O3=0.5600D+00*HCHO+...
     &                     +    9.0000D-02 * RXRAT(   144 )      ! OLI+O3=0.9000D-01*HCHO+...
     &                     +    7.9000D-01 * RXRAT(   147 )      ! LIMAL+O3=0.7900D+00*HCHO+...
     &                     +    1.0000D-01 * RXRAT(   150 )      ! MVK+O3=0.1000D+00*HCHO+...
     &                     +    8.0000D-02 * RXRAT(   151 )      ! UALD+O3=0.8000D-01*HCHO+...
     &                     +    5.0000D-02 * RXRAT(   152 )      ! DCB1+O3=0.5000D-01*HCHO+...
     &                     +    5.0000D-02 * RXRAT(   153 )      ! DCB2+O3=0.5000D-01*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   165 )      ! MACR+NO3=0.6800D+00*HCHO+...
     &                     +    3.3200D-01 * RXRAT(   166 )      ! UALD+NO3=0.3320D+00*HCHO+...
     &                     +                 RXRAT(   181 )      ! MO2+NO=HCHO+NO2+HO2
     &                     +    1.8000D-02 * RXRAT(   184 )      ! HC5P+NO=0.1800D-01*HCHO+...
     &                     +    1.6000D+00 * RXRAT(   185 )      ! ETEP+NO=0.1600D+01*HCHO+...
     &                     +    7.8000D-01 * RXRAT(   186 )      ! OLTP+NO=0.7800D+00*HCHO+...
     &                     +    2.8000D-01 * RXRAT(   196 )      ! LIMP1+NO=0.2800D+00*HCHO+...
     &                     +    9.5000D-01 * RXRAT(   200 )      ! PINALP+NO=0.9500D+00*HCHO+...
     &                     +    9.4000D-01 * RXRAT(   201 )      ! LIMALP+NO=0.9400D+00*HCHO+...
     &                     +                 RXRAT(   204 )      ! ACTP+NO=HCHO+NO2+ACO3
     &                     +    3.3000D-01 * RXRAT(   205 )      ! MEKP+NO=0.3300D+00*HCHO+...
     &                     +    6.5000D-01 * RXRAT(   207 )      ! MACP+NO=0.6500D+00*HCHO+...
     &                     +    5.0000D-01 * RXRAT(   208 )      ! MCP+NO=0.5000D+00*HCHO+...
     &                     +    3.0000D-01 * RXRAT(   209 )      ! MVKP+NO=0.3000D+00*HCHO+...
     &                     +    3.0000D-02 * RXRAT(   210 )      ! UALP+NO=0.3000D-01*HCHO+...
     &                     +    2.8700D-01 * RXRAT(   217 )      ! OLND+NO=0.2870D+00*HCHO+...
     &                     +    1.5000D-01 * RXRAT(   247 )      ! ACTP+HO2=0.1500D+00*HCHO+...
     &                     +    1.3700D+00 * RXRAT(   262 )      ! MO2+MO2=0.1370D+01*HCHO+...
     &                     +    7.5000D-01 * RXRAT(   263 )      ! ETHP+MO2=0.7500D+00*HCHO+...
     &                     +    8.2700D-01 * RXRAT(   264 )      ! HC3P+MO2=0.8270D+00*HCHO+...
     &                     +    7.7700D-01 * RXRAT(   265 )      ! HC5P+MO2=0.7770D+00*HCHO+...
     &                     +    1.9500D+00 * RXRAT(   266 )      ! ETEP+MO2=0.1950D+01*HCHO+...
     &                     +    1.5000D+00 * RXRAT(   267 )      ! OLTP+MO2=0.1500D+01*HCHO+...
     &                     +    7.5000D-01 * RXRAT(   268 )      ! OLIP+MO2=0.7500D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   269 )      ! BENP+MO2=0.6800D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   270 )      ! TOLP+MO2=0.6800D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   271 )      ! XYMP+MO2=0.6800D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   272 )      ! XYEP+MO2=0.6800D+00*HCHO+...
     &                     +    1.3100D+00 * RXRAT(   273 )      ! ISOP+MO2=0.1310D+01*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   274 )      ! APIP1+MO2=0.6800D+00*HCHO+...
     &                     +    7.5000D-01 * RXRAT(   275 )      ! APIP2+MO2=0.7500D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   276 )      ! APINP1+MO2=0.6800D+00*HCHO+...
     &                     +    7.5000D-01 * RXRAT(   277 )      ! APINP2+MO2=0.7500D+00*HCHO+...
     &                     +                 RXRAT(   278 )      ! LIMP1+MO2=HCHO+HO2+0.4200D+...
     &                     +    7.5000D-01 * RXRAT(   279 )      ! LIMP2+MO2=0.7500D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   280 )      ! LIMNP1+MO2=0.6800D+00*HCHO+...
     &                     +    7.5000D-01 * RXRAT(   281 )      ! LIMNP2+MO2=0.7500D+00*HCHO+...
     &                     +                 RXRAT(   282 )      ! ACO3+MO2=HCHO+0.9000D+...
     &                     +                 RXRAT(   283 )      ! RCO3+MO2=HCHO+0.9000D+...
     &                     +    1.5000D+00 * RXRAT(   284 )      ! ACTP+MO2=0.1500D+01*HCHO+...
     &                     +                 RXRAT(   285 )      ! MEKP+MO2=HCHO+0.8340D+...
     &                     +    7.5000D-01 * RXRAT(   286 )      ! KETP+MO2=0.7500D+00*HCHO+...
     &                     +    1.6600D+00 * RXRAT(   287 )      ! MACP+MO2=0.1660D+01*HCHO+...
     &                     +    1.5000D+00 * RXRAT(   288 )      ! MCP+MO2=0.1500D+01*HCHO+...
     &                     +    1.5000D+00 * RXRAT(   289 )      ! MVKP+MO2=0.1500D+01*HCHO+...
     &                     +    7.7300D-01 * RXRAT(   290 )      ! UALP+MO2=0.7730D+00*HCHO+...
     &                     +                 RXRAT(   291 )      ! BALP+MO2=HCHO+BAL1+HO2
     &                     +                 RXRAT(   292 )      ! BAL1+MO2=HCHO+BAL2+HO2
     &                     +                 RXRAT(   293 )      ! ADDC+MO2=HCHO+0.2000D+...
     &                     +                 RXRAT(   294 )      ! MCTP+MO2=HCHO+MCTO+HO2
     &                     +                 RXRAT(   295 )      ! ORAP+MO2=HCHO+HO2+GLY
     &                     +                 RXRAT(   296 )      ! OLNN+MO2=HCHO+0.2000D+...
     &                     +    9.6500D-01 * RXRAT(   297 )      ! OLND+MO2=0.9650D+00*HCHO+...
     &                     +                 RXRAT(   298 )      ! ADCN+MO2=HCHO+0.7000D+...
     &                     +                 RXRAT(   299 )      ! XO2+MO2=HCHO+HO2
     &                     +    1.3000D-01 * RXRAT(   301 )      ! HC3P+ACO3=0.1300D+00*HCHO+...
     &                     +    4.2000D-02 * RXRAT(   302 )      ! HC5P+ACO3=0.4200D-01*HCHO+...
     &                     +    1.6000D+00 * RXRAT(   303 )      ! ETEP+ACO3=0.1600D+01*HCHO+...
     &                     +                 RXRAT(   304 )      ! OLTP+ACO3=HCHO+0.5000D+...
     &                     +    1.0480D+00 * RXRAT(   310 )      ! ISOP+ACO3=0.1048D+01*HCHO+...
     &                     +    3.2000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.3200D+00*HCHO+...
     &                     +                 RXRAT(   321 )      ! ACTP+ACO3=HCHO+0.5000D+...
     &                     +    3.3000D-01 * RXRAT(   322 )      ! MEKP+ACO3=0.3300D+00*HCHO+...
     &                     +                 RXRAT(   324 )      ! MACP+ACO3=HCHO+0.5000D+...
     &                     +                 RXRAT(   325 )      ! MCP+ACO3=HCHO+0.5000D+...
     &                     +                 RXRAT(   326 )      ! MVKP+ACO3=HCHO+0.5000D+...
     &                     +    3.0000D-02 * RXRAT(   327 )      ! UALP+ACO3=0.3000D-01*HCHO+...
     &                     +    2.8700D-01 * RXRAT(   334 )      ! OLND+ACO3=0.2870D+00*HCHO+...
     &                     +                 RXRAT(   338 )      ! MO2+NO3=HCHO+HO2+NO2
     &                     +    2.4000D-02 * RXRAT(   341 )      ! HC5P+NO3=0.2400D-01*HCHO+...
     &                     +    1.6000D+00 * RXRAT(   342 )      ! ETEP+NO3=0.1600D+01*HCHO+...
     &                     +    7.9000D-01 * RXRAT(   343 )      ! OLTP+NO3=0.7900D+00*HCHO+...
     &                     +    7.5000D-01 * RXRAT(   349 )      ! ISOP+NO3=0.7500D+00*HCHO+...
     &                     +    3.8500D-01 * RXRAT(   351 )      ! LIMP1+NO3=0.3850D+00*HCHO+...
     &                     +                 RXRAT(   354 )      ! ACTP+NO3=HCHO+NO2+ACO3
     &                     +    3.3000D-01 * RXRAT(   355 )      ! MEKP+NO3=0.3300D+00*HCHO+...
     &                     +                 RXRAT(   357 )      ! MACP+NO3=HCHO+0.5380D+...
     &                     +                 RXRAT(   358 )      ! MCP+NO3=HCHO+HO2+NO2+HKET
     &                     +    3.0000D-01 * RXRAT(   359 )      ! MVKP+NO3=0.3000D+00*HCHO+...
     &                     +    3.0000D-02 * RXRAT(   360 )      ! UALP+NO3=0.3000D-01*HCHO+...
     &                     +    2.8700D-01 * RXRAT(   367 )      ! OLND+NO3=0.2870D+00*HCHO+...
     &                     +    2.0200D-01 * RXRAT(   370 )      ! OLNN+OLND=0.2020D+00*HCHO+...
     &                     +    5.0400D-01 * RXRAT(   371 )      ! OLND+OLND=0.5040D+00*HCHO+...
     &                     +    4.8000D-01 * RXRAT(   377 )      ! APIP2+ISOP=0.4800D+00*HCHO+...
     &                     +    4.8000D-01 * RXRAT(   380 )      ! LIMP2+ISOP=0.4800D+00*HCHO+...
     &                     +    4.8000D-01 * RXRAT(   383 )      ! APINP2+ISOP=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   386 )      ! LIMNP2+ISOP=0.4800D+...
     &                     +    7.2000D-01 * RXRAT(   391 )      ! ACRO+O3=0.7200D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   392 )      ! ACRO+NO3=0.6800D+00*HCHO+...
     &                     +    1.5000D-01 * RXRAT(   393 )      ! ACRO=0.1500D+00*HCHO+...
     &                     +    8.9500D-01 * RXRAT(   395 )      ! BDE13P+NO=0.8950D+00*HCHO+...
     &                     +    9.2500D-01 * RXRAT(   396 )      ! BDE13P+NO3=0.9250D+00*HCHO+...
     &                     +    1.1430D+00 * RXRAT(   398 )      ! BDE13P+MO2=0.1143D+01*HCHO+...
     &                     +    7.4000D-01 * RXRAT(   399 )      ! BDE13P+ACO3=0.7400D+...
     &                     +    8.3000D-01 * RXRAT(   400 )      ! BDE13+O3=0.8300D+00*HCHO+...
     &                     +    6.8000D-01 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.6800D+...
     &                     +    6.8000D-01 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.6800D+...
     &                     +    6.8000D-01 * RXRAT(   500 )      ! NAPHP+MO2=0.6800D+00*HCHO+...
      LOSS(  HCHO        ) =                 RXRAT(    10 )      ! HCHO=CO
     &                     +                 RXRAT(    11 )      ! HCHO=0.2000D+01*HO2+CO
     &                     +                 RXRAT(   107 )      ! HCHO+HO=HO2+CO
     &                     +                 RXRAT(   162 )      ! HCHO+NO3=HO2+CO+HNO3

      PROD(  CO          ) =                 RXRAT(    10 )      ! HCHO=CO
     &                     +                 RXRAT(    11 )      ! HCHO=CO+0.2000D+01*HO2
     &                     +                 RXRAT(    12 )      ! ACD=CO+MO2+HO2
     &                     +                 RXRAT(    13 )      ! ALD=CO+ETHP+HO2
     &                     +                 RXRAT(    15 )      ! ACT=CO+0.2000D+01*MO2
     &                     +    1.2200D+00 * RXRAT(    16 )      ! UALD=0.1220D+01*CO+0.7840D+...
     &                     +                 RXRAT(    17 )      ! PINAL=CO+HC10P+HO2
     &                     +                 RXRAT(    18 )      ! LIMAL=CO+HC10P+HO2
     &                     +    1.0000D-01 * RXRAT(    19 )      ! MEK=0.1000D+00*CO+ETHP+...
     &                     +    5.0000D-01 * RXRAT(    20 )      ! KET=0.5000D+00*CO+0.5000D+...
     &                     +    6.7000D-01 * RXRAT(    22 )      ! MACR=0.6700D+00*CO+0.6600D+...
     &                     +    7.0000D-01 * RXRAT(    23 )      ! MVK=0.7000D+00*CO+0.3000D+...
     &                     +    2.0000D+00 * RXRAT(    24 )      ! GLY=0.2000D+01*CO
     &                     +                 RXRAT(    25 )      ! GLY=CO+HCHO
     &                     +    2.0000D+00 * RXRAT(    26 )      ! GLY=0.2000D+01*CO+0.2000D+...
     &                     +                 RXRAT(    27 )      ! MGLY=CO+ACO3+HO2
     &                     +                 RXRAT(    28 )      ! DCB1=CO+0.2500D+00*ACO3+...
     &                     +                 RXRAT(    29 )      ! DCB2=CO+0.2500D+00*ACO3+...
     &                     +                 RXRAT(    30 )      ! BALD=CO+HO2+CHO
     &                     +    3.5000D-01 * RXRAT(    82 )      ! ACE+HO=0.3500D+00*CO+...
     &                     +    1.4000D-01 * RXRAT(    87 )      ! ISO+O3=0.1400D+00*CO+...
     &                     +                 RXRAT(   101 )      ! ISON+HO=CO+0.1200D+00*NO2
     &                     +                 RXRAT(   102 )      ! ISON+NO3=CO
     &                     +                 RXRAT(   107 )      ! HCHO+HO=CO+HO2
     &                     +    2.0000D+00 * RXRAT(   117 )      ! GLY+HO=0.2000D+01*CO+HO2
     &                     +                 RXRAT(   118 )      ! MGLY+HO=CO+ACO3
     &                     +    3.3000D-01 * RXRAT(   119 )      ! DCB1+HO=0.3300D+00*CO+...
     &                     +    3.3000D-01 * RXRAT(   120 )      ! DCB2+HO=0.3300D+00*CO+...
     &                     +    1.1000D-01 * RXRAT(   121 )      ! DCB3+HO=0.1100D+00*CO+...
     &                     +    4.3000D-01 * RXRAT(   142 )      ! ETE+O3=0.4300D+00*CO+...
     &                     +    4.3000D-01 * RXRAT(   143 )      ! OLT+O3=0.4300D+00*CO+...
     &                     +    3.7000D-01 * RXRAT(   144 )      ! OLI+O3=0.3700D+00*CO+...
     &                     +    1.4000D-01 * RXRAT(   145 )      ! API+O3=0.1400D+00*CO+...
     &                     +    1.4000D-01 * RXRAT(   146 )      ! LIM+O3=0.1400D+00*CO+...
     &                     +    2.0000D-01 * RXRAT(   147 )      ! LIMAL+O3=0.2000D+00*CO+...
     &                     +    2.2000D-01 * RXRAT(   149 )      ! MACR+O3=0.2200D+00*CO+...
     &                     +    5.6000D-01 * RXRAT(   150 )      ! MVK+O3=0.5600D+00*CO+...
     &                     +    2.4300D-01 * RXRAT(   151 )      ! UALD+O3=0.2430D+00*CO+...
     &                     +    1.5000D+00 * RXRAT(   152 )      ! DCB1+O3=0.1500D+01*CO+HO2+...
     &                     +    1.5000D+00 * RXRAT(   153 )      ! DCB2+O3=0.1500D+01*CO+HO2+...
     &                     +    1.5000D+00 * RXRAT(   154 )      ! DCB3+O3=0.1500D+01*CO+HO2+...
     &                     +                 RXRAT(   162 )      ! HCHO+NO3=CO+HO2+HNO3
     &                     +    6.6800D-01 * RXRAT(   166 )      ! UALD+NO3=0.6680D+00*CO+XO2+...
     &                     +    2.0000D+00 * RXRAT(   167 )      ! GLY+NO3=0.2000D+01*CO+HO2+HNO3
     &                     +                 RXRAT(   168 )      ! MGLY+NO3=CO+ACO3+HNO3
     &                     +    6.5000D-01 * RXRAT(   207 )      ! MACP+NO=0.6500D+00*CO+...
     &                     +    6.1000D-01 * RXRAT(   210 )      ! UALP+NO=0.6100D+00*CO+NO2+...
     &                     +    5.0000D-01 * RXRAT(   287 )      ! MACP+MO2=0.5000D+00*CO+...
     &                     +    3.0500D-01 * RXRAT(   290 )      ! UALP+MO2=0.3050D+00*CO+HO2+...
     &                     +    5.0000D-01 * RXRAT(   324 )      ! MACP+ACO3=0.5000D+00*CO+...
     &                     +    5.0000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.5000D+00*CO+...
     &                     +                 RXRAT(   357 )      ! MACP+NO3=CO+0.5380D+...
     &                     +    6.1000D-01 * RXRAT(   360 )      ! UALP+NO3=0.6100D+00*CO+NO2+...
     &                     +    8.4000D-01 * RXRAT(   391 )      ! ACRO+O3=0.8400D+00*CO+...
     &                     +                 RXRAT(   393 )      ! ACRO=CO+0.4770D+00*HO2+...
     &                     +    6.3000D-01 * RXRAT(   400 )      ! BDE13+O3=0.6300D+00*CO+...
     &                     +    4.3000D-01 * RXRAT(   405 )      ! FURANONE+HO=0.4300D+00*CO+...
      LOSS(  CO          ) =                 RXRAT(    74 )      ! CO+HO=HO2

      PROD(  ACD         ) =                 RXRAT(   127 )      ! EOH+HO=ACD+HO2
     &                     +    1.8400D-01 * RXRAT(   128 )      ! ROH+HO=0.1840D+00*ACD+...
     &                     +    1.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.1000D-01*ACD+...
     &                     +    4.5700D-01 * RXRAT(   144 )      ! OLI+O3=0.4570D+00*ACD+...
     &                     +    4.2000D-01 * RXRAT(   151 )      ! UALD+O3=0.4200D+00*ACD+...
     &                     +                 RXRAT(   182 )      ! ETHP+NO=ACD+NO2+HO2
     &                     +    5.0400D-01 * RXRAT(   183 )      ! HC3P+NO=0.5040D+00*ACD+...
     &                     +    4.5000D-02 * RXRAT(   184 )      ! HC5P+NO=0.4500D-01*ACD+...
     &                     +    1.2000D-02 * RXRAT(   186 )      ! OLTP+NO=0.1200D-01*ACD+...
     &                     +    8.1000D-01 * RXRAT(   187 )      ! OLIP+NO=0.8100D+00*ACD+...
     &                     +    7.5000D-01 * RXRAT(   263 )      ! ETHP+MO2=0.7500D+00*ACD+...
     &                     +                 RXRAT(   300 )      ! ETHP+ACO3=ACD+0.5000D+...
     &                     +                 RXRAT(   339 )      ! ETHP+NO3=ACD+NO2+HO2
     &                     +    5.1900D-01 * RXRAT(   340 )      ! HC3P+NO3=0.5190D+00*ACD+...
     &                     +    4.8000D-02 * RXRAT(   341 )      ! HC5P+NO3=0.4800D-01*ACD+...
     &                     +    2.0000D-02 * RXRAT(   343 )      ! OLTP+NO3=0.2000D-01*ACD+...
     &                     +    8.5000D-01 * RXRAT(   344 )      ! OLIP+NO3=0.8500D+00*ACD+...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*ACD+...
      LOSS(  ACD         ) =                 RXRAT(    12 )      ! ACD=HO2+MO2+CO
     &                     +                 RXRAT(   108 )      ! ACD+HO=ACO3
     &                     +                 RXRAT(   163 )      ! ACD+NO3=ACO3+HNO3

      PROD(  MO2         ) =                 RXRAT(    12 )      ! ACD=MO2+HO2+CO
     &                     +                 RXRAT(    14 )      ! ACT=MO2+ACO3
     &                     +    2.0000D+00 * RXRAT(    15 )      ! ACT=0.2000D+01*MO2+CO
     &                     +    1.0000D-01 * RXRAT(    19 )      ! MEK=0.1000D+00*MO2+ETHP+...
     &                     +    3.0000D-01 * RXRAT(    23 )      ! MVK=0.3000D+00*MO2+0.3000D+...
     &                     +                 RXRAT(    34 )      ! PAA=MO2+HO
     &                     +                 RXRAT(    37 )      ! PAN=MO2+NO3
     &                     +                 RXRAT(    75 )      ! HO=MO2
     &                     +    8.0000D-02 * RXRAT(    87 )      ! ISO+O3=0.8000D-01*MO2+...
     &                     +    2.0000D-02 * RXRAT(    89 )      ! ISO+HO=0.2000D-01*MO2+ISOP
     &                     +    1.0000D-01 * RXRAT(    94 )      ! INO2+NO=0.1000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(    95 )      ! IPN+HO2=0.7000D+00*MO2+...
     &                     +    6.5000D-01 * RXRAT(   130 )      ! OP1+HO=0.6500D+00*MO2+...
     &                     +    6.4000D-01 * RXRAT(   135 )      ! ORA2+HO=0.6400D+00*MO2+...
     &                     +    8.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.8000D-01*MO2+...
     &                     +    3.2000D-01 * RXRAT(   144 )      ! OLI+O3=0.3200D+00*MO2+...
     &                     +    8.0000D-03 * RXRAT(   151 )      ! UALD+O3=0.8000D-02*MO2+...
     &                     +    1.3100D-01 * RXRAT(   183 )      ! HC3P+NO=0.1310D+00*MO2+...
     &                     +    5.1000D-02 * RXRAT(   184 )      ! HC5P+NO=0.5100D-01*MO2+...
     &                     +                 RXRAT(   202 )      ! ACO3+NO=MO2+NO2
     &                     +    6.5000D-01 * RXRAT(   207 )      ! MACP+NO=0.6500D+00*MO2+...
     &                     +    4.4000D-01 * RXRAT(   245 )      ! ACO3+HO2=0.4400D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   300 )      ! ETHP+ACO3=0.5000D+00*MO2+...
     &                     +    5.8000D-01 * RXRAT(   301 )      ! HC3P+ACO3=0.5800D+00*MO2+...
     &                     +    5.1800D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.5180D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   303 )      ! ETEP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   304 )      ! OLTP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   305 )      ! OLIP+ACO3=0.5000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   306 )      ! BENP+ACO3=0.7000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.7000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.7000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.7000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   310 )      ! ISOP+ACO3=0.5000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   311 )      ! APIP1+ACO3=0.7000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   312 )      ! APIP2+ACO3=0.5000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   313 )      ! APINP1+ACO3=0.7000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   314 )      ! APINP2+ACO3=0.5000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.7000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   316 )      ! LIMP2+ACO3=0.5000D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   317 )      ! LIMNP1+ACO3=0.7000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   318 )      ! LIMNP2+ACO3=0.5000D+00*MO2+...
     &                     +    2.0000D+00 * RXRAT(   319 )      ! ACO3+ACO3=0.2000D+01*MO2
     &                     +                 RXRAT(   320 )      ! RCO3+ACO3=MO2+ETHP
     &                     +    5.0000D-01 * RXRAT(   321 )      ! ACTP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   322 )      ! MEKP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   323 )      ! KETP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   324 )      ! MACP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   325 )      ! MCP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   326 )      ! MVKP+ACO3=0.5000D+00*MO2+...
     &                     +    5.0000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.5000D+00*MO2+...
     &                     +                 RXRAT(   328 )      ! BALP+ACO3=MO2+BAL1
     &                     +                 RXRAT(   329 )      ! BAL1+ACO3=MO2+BAL2
     &                     +                 RXRAT(   330 )      ! ADDC+ACO3=MO2+0.2000D+...
     &                     +                 RXRAT(   331 )      ! MCTP+ACO3=MO2+HO2+MCTO
     &                     +                 RXRAT(   332 )      ! ORAP+ACO3=MO2+GLY
     &                     +                 RXRAT(   333 )      ! OLNN+ACO3=MO2+HO2+ONIT
     &                     +    5.0000D-01 * RXRAT(   334 )      ! OLND+ACO3=0.5000D+00*MO2+...
     &                     +                 RXRAT(   335 )      ! ADCN+ACO3=MO2+HO2+0.7000D+...
     &                     +                 RXRAT(   336 )      ! XO2+ACO3=MO2
     &                     +    1.4000D-01 * RXRAT(   340 )      ! HC3P+NO3=0.1400D+00*MO2+...
     &                     +    5.5000D-02 * RXRAT(   341 )      ! HC5P+NO3=0.5500D-01*MO2+...
     &                     +                 RXRAT(   352 )      ! ACO3+NO3=MO2+NO2
     &                     +    2.7000D-02 * RXRAT(   393 )      ! ACRO=0.2700D-01*MO2+...
     &                     +    7.0000D-01 * RXRAT(   399 )      ! BDE13P+ACO3=0.7000D+00*MO2+...
     &                     +    1.7000D-01 * RXRAT(   403 )      ! FURANO2+NO=0.1700D+00*MO2+...
     &                     +    8.0000D-02 * RXRAT(   404 )      ! FURANO2+HO2=0.8000D-01*MO2+...
     &                     +    3.4000D-01 * RXRAT(   405 )      ! FURANONE+HO=0.3400D+00*MO2+...
     &                     +    7.0000D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.7000D+...
     &                     +    7.0000D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.7000D+...
     &                     +    7.0000D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.7000D+00*MO2+...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*MO2+...
      LOSS(  MO2         ) =                 RXRAT(   181 )      ! MO2+NO=HO2+NO2+HCHO
     &                     +                 RXRAT(   223 )      ! MO2+HO2=OP1
     &                     +    2.0000D+00 * RXRAT(   262 )      ! MO2+MO2=0.7400D+00*HO2+...
     &                     +                 RXRAT(   263 )      ! MO2+ETHP=HO2+0.7500D+...
     &                     +    9.2000D-01 * RXRAT(   264 )      ! MO2+HC3P=0.8000D-01*MO2+...
     &                     +    9.8200D-01 * RXRAT(   265 )      ! MO2+HC5P=0.1800D-01*MO2+...
     &                     +                 RXRAT(   266 )      ! MO2+ETEP=HO2+0.1950D+...
     &                     +                 RXRAT(   267 )      ! MO2+OLTP=HO2+0.1500D+...
     &                     +                 RXRAT(   268 )      ! MO2+OLIP=HO2+0.7500D+...
     &                     +                 RXRAT(   269 )      ! MO2+BENP=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   270 )      ! MO2+TOLP=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   271 )      ! MO2+XYMP=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   272 )      ! MO2+XYEP=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   273 )      ! MO2+ISOP=HO2+0.1310D+...
     &                     +                 RXRAT(   274 )      ! MO2+APIP1=HO2+0.6800D+...
     &                     +                 RXRAT(   275 )      ! MO2+APIP2=HO2+0.7500D+...
     &                     +                 RXRAT(   276 )      ! MO2+APINP1=0.3700D+00*HO2+...
     &                     +                 RXRAT(   277 )      ! MO2+APINP2=0.7500D+00*HO2+...
     &                     +                 RXRAT(   278 )      ! MO2+LIMP1=HO2+HCHO+0.4200D+...
     &                     +                 RXRAT(   279 )      ! MO2+LIMP2=HO2+0.7500D+...
     &                     +                 RXRAT(   280 )      ! MO2+LIMNP1=0.3700D+00*HO2+...
     &                     +                 RXRAT(   281 )      ! MO2+LIMNP2=0.7500D+00*HO2+...
     &                     +    1.0000D-01 * RXRAT(   282 )      ! MO2+ACO3=0.9000D+00*MO2+...
     &                     +    1.0000D-01 * RXRAT(   283 )      ! MO2+RCO3=0.9000D+00*MO2+...
     &                     +                 RXRAT(   284 )      ! MO2+ACTP=0.5000D+00*HO2+...
     &                     +                 RXRAT(   285 )      ! MO2+MEKP=0.8340D+00*HO2+...
     &                     +                 RXRAT(   286 )      ! MO2+KETP=HO2+0.7500D+...
     &                     +    7.5000D-01 * RXRAT(   287 )      ! MO2+MACP=0.2500D+00*MO2+...
     &                     +                 RXRAT(   288 )      ! MO2+MCP=NO2+HO2+0.1500D+...
     &                     +                 RXRAT(   289 )      ! MO2+MVKP=HO2+0.1160D+...
     &                     +                 RXRAT(   290 )      ! MO2+UALP=HO2+0.3050D+00*CO+...
     &                     +                 RXRAT(   291 )      ! MO2+BALP=HO2+BAL1+HCHO
     &                     +                 RXRAT(   292 )      ! MO2+BAL1=HO2+BAL2+HCHO
     &                     +                 RXRAT(   293 )      ! MO2+ADDC=0.2000D+01*HO2+...
     &                     +                 RXRAT(   294 )      ! MO2+MCTP=HO2+MCTO+HCHO
     &                     +                 RXRAT(   295 )      ! MO2+ORAP=HCHO+HO2+GLY
     &                     +                 RXRAT(   296 )      ! MO2+OLNN=0.2000D+01*HO2+...
     &                     +                 RXRAT(   297 )      ! MO2+OLND=0.5000D+00*HO2+...
     &                     +                 RXRAT(   298 )      ! MO2+ADCN=HO2+0.7000D+...
     &                     +                 RXRAT(   299 )      ! MO2+XO2=HO2+HCHO
     &                     +                 RXRAT(   338 )      ! MO2+NO3=HO2+HCHO+NO2
     &                     +                 RXRAT(   398 )      ! MO2+BDE13P=0.3200D+00*MOH+...
     &                     +                 RXRAT(   488 )      ! MO2+VROCP6AROP=0.6800D+...
     &                     +                 RXRAT(   494 )      ! MO2+VROCP5AROP=0.6800D+...
     &                     +                 RXRAT(   500 )      ! MO2+NAPHP=0.6800D+00*HCHO+...

      PROD(  ALD         ) =    4.3400D-01 * RXRAT(    16 )      ! UALD=0.4340D+00*ALD+...
     &                     +                 RXRAT(    32 )      ! OP2=ALD+HO2+HO
     &                     +                 RXRAT(    33 )      ! OPB=ALD+HO2+HO
     &                     +    2.0000D-01 * RXRAT(    35 )      ! ONIT=0.2000D+00*ALD+NO2+...
     &                     +    4.0000D-01 * RXRAT(   119 )      ! DCB1+HO=0.4000D+00*ALD+...
     &                     +    7.1900D-01 * RXRAT(   128 )      ! ROH+HO=0.7190D+00*ALD+HO2+...
     &                     +                 RXRAT(   129 )      ! ETEG+HO=ALD+HO2
     &                     +    8.0000D-02 * RXRAT(   131 )      ! OP2+HO=0.8000D-01*ALD+...
     &                     +    8.0000D-02 * RXRAT(   132 )      ! OPB+HO=0.8000D-01*ALD+...
     &                     +    4.4000D-01 * RXRAT(   143 )      ! OLT+O3=0.4400D+00*ALD+...
     &                     +    7.3000D-01 * RXRAT(   144 )      ! OLI+O3=0.7300D+00*ALD+...
     &                     +    1.0000D-01 * RXRAT(   150 )      ! MVK+O3=0.1000D+00*ALD+...
     &                     +    3.3200D-01 * RXRAT(   166 )      ! UALD+NO3=0.3320D+00*ALD+...
     &                     +    1.3200D-01 * RXRAT(   183 )      ! HC3P+NO=0.1320D+00*ALD+...
     &                     +    2.0300D-01 * RXRAT(   184 )      ! HC5P+NO=0.2030D+00*ALD+...
     &                     +    2.0000D-01 * RXRAT(   185 )      ! ETEP+NO=0.2000D+00*ALD+NO2+...
     &                     +    4.4000D-01 * RXRAT(   186 )      ! OLTP+NO=0.4400D+00*ALD+...
     &                     +    6.8000D-01 * RXRAT(   187 )      ! OLIP+NO=0.6800D+00*ALD+...
     &                     +    4.6000D-01 * RXRAT(   206 )      ! KETP+NO=0.4600D+00*ALD+...
     &                     +    7.0000D-01 * RXRAT(   209 )      ! MVKP+NO=0.7000D+00*ALD+...
     &                     +    2.7000D-01 * RXRAT(   210 )      ! UALP+NO=0.2700D+00*ALD+NO2+...
     &                     +    1.2400D+00 * RXRAT(   217 )      ! OLND+NO=0.1240D+01*ALD+...
     &                     +    1.9800D-01 * RXRAT(   264 )      ! HC3P+MO2=0.1980D+00*ALD+...
     &                     +    2.5100D-01 * RXRAT(   265 )      ! HC5P+MO2=0.2510D+00*ALD+...
     &                     +    1.5000D-01 * RXRAT(   266 )      ! ETEP+MO2=0.1500D+00*ALD+...
     &                     +    7.0500D-01 * RXRAT(   267 )      ! OLTP+MO2=0.7050D+00*ALD+...
     &                     +    1.2800D+00 * RXRAT(   268 )      ! OLIP+MO2=0.1280D+01*ALD+...
     &                     +    2.3000D-02 * RXRAT(   273 )      ! ISOP+MO2=0.2300D-01*ALD+...
     &                     +    1.7500D+00 * RXRAT(   289 )      ! MVKP+MO2=0.1750D+01*ALD+...
     &                     +    2.0300D-01 * RXRAT(   290 )      ! UALP+MO2=0.2030D+00*ALD+...
     &                     +    9.3000D-01 * RXRAT(   297 )      ! OLND+MO2=0.9300D+00*ALD+...
     &                     +    2.7300D-01 * RXRAT(   301 )      ! HC3P+ACO3=0.2730D+00*ALD+...
     &                     +    3.8100D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.3810D+00*ALD+...
     &                     +    2.0000D-01 * RXRAT(   303 )      ! ETEP+ACO3=0.2000D+00*ALD+...
     &                     +    9.4000D-01 * RXRAT(   304 )      ! OLTP+ACO3=0.9400D+00*ALD+...
     &                     +    1.7100D+00 * RXRAT(   305 )      ! OLIP+ACO3=0.1710D+01*ALD+...
     &                     +    2.3000D+00 * RXRAT(   326 )      ! MVKP+ACO3=0.2300D+01*ALD+...
     &                     +    2.7000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.2700D+00*ALD+...
     &                     +    1.2400D+00 * RXRAT(   334 )      ! OLND+ACO3=0.1240D+01*ALD+...
     &                     +    1.4700D-01 * RXRAT(   340 )      ! HC3P+NO3=0.1470D+00*ALD+...
     &                     +    2.4100D-01 * RXRAT(   341 )      ! HC5P+NO3=0.2410D+00*ALD+...
     &                     +    2.0000D-01 * RXRAT(   342 )      ! ETEP+NO3=0.2000D+00*ALD+...
     &                     +    4.7000D-01 * RXRAT(   343 )      ! OLTP+NO3=0.4700D+00*ALD+...
     &                     +    7.2000D-01 * RXRAT(   344 )      ! OLIP+NO3=0.7200D+00*ALD+...
     &                     +    3.1000D-02 * RXRAT(   349 )      ! ISOP+NO3=0.3100D-01*ALD+...
     &                     +                 RXRAT(   350 )      ! APIP1+NO3=ALD+NO2+HO2+KET
     &                     +    7.0000D-01 * RXRAT(   359 )      ! MVKP+NO3=0.7000D+00*ALD+...
     &                     +    2.7000D-01 * RXRAT(   360 )      ! UALP+NO3=0.2700D+00*ALD+...
     &                     +    1.2400D+00 * RXRAT(   367 )      ! OLND+NO3=0.1240D+01*ALD+...
     &                     +    6.4000D-01 * RXRAT(   370 )      ! OLNN+OLND=0.6400D+00*ALD+...
     &                     +    1.2100D+00 * RXRAT(   371 )      ! OLND+OLND=0.1210D+01*ALD+...
     &                     +                 RXRAT(   406 )      ! FURAN+O3=ALD+0.2000D-01*HO
     &                     +    3.8700D-01 * RXRAT(   408 )      ! PROG+HO=0.3870D+00*ALD+...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*ALD+...
      LOSS(  ALD         ) =                 RXRAT(    13 )      ! ALD=HO2+ETHP+CO
     &                     +                 RXRAT(   109 )      ! ALD+HO=RCO3
     &                     +                 RXRAT(   164 )      ! ALD+NO3=RCO3+HNO3

      PROD(  ETHP        ) =                 RXRAT(    13 )      ! ALD=ETHP+HO2+CO
     &                     +                 RXRAT(    19 )      ! MEK=ETHP+0.1000D+00*MO2+...
     &                     +    1.5000D+00 * RXRAT(    20 )      ! KET=0.1500D+01*ETHP+...
     &                     +                 RXRAT(    76 )      ! ETH+HO=ETHP
     &                     +    6.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.6000D-01*ETHP+...
     &                     +    7.0000D-02 * RXRAT(   144 )      ! OLI+O3=0.7000D-01*ETHP+...
     &                     +    4.8000D-02 * RXRAT(   183 )      ! HC3P+NO=0.4800D-01*ETHP+...
     &                     +    2.3100D-01 * RXRAT(   184 )      ! HC5P+NO=0.2310D+00*ETHP+...
     &                     +                 RXRAT(   203 )      ! RCO3+NO=ETHP+NO2
     &                     +    4.4000D-01 * RXRAT(   246 )      ! RCO3+HO2=0.4400D+00*ETHP+...
     &                     +    2.6000D-02 * RXRAT(   264 )      ! HC3P+MO2=0.2600D-01*ETHP+...
     &                     +    1.4000D-01 * RXRAT(   265 )      ! HC5P+MO2=0.1400D+00*ETHP+...
     &                     +    2.6000D-02 * RXRAT(   301 )      ! HC3P+ACO3=0.2600D-01*ETHP+...
     &                     +    1.4000D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.1400D+00*ETHP+...
     &                     +                 RXRAT(   320 )      ! RCO3+ACO3=ETHP+MO2
     &                     +    2.0000D+00 * RXRAT(   337 )      ! RCO3+RCO3=0.2000D+01*ETHP
     &                     +    5.0300D-01 * RXRAT(   340 )      ! HC3P+NO3=0.5030D+00*ETHP+...
     &                     +    2.8000D-01 * RXRAT(   341 )      ! HC5P+NO3=0.2800D+00*ETHP+...
     &                     +                 RXRAT(   353 )      ! RCO3+NO3=ETHP+NO2
     &                     +                 RXRAT(   373 )      ! XO2+RCO3=ETHP
     &                     +    8.5200D-01 * RXRAT(   388 )      ! VROCIOXY+HO=0.8520D+...
     &                     +                 RXRAT(   389 )      ! SLOWROC+HO=ETHP+0.1010D-...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*ETHP+...
      LOSS(  ETHP        ) =                 RXRAT(   182 )      ! ETHP+NO=HO2+NO2+ACD
     &                     +                 RXRAT(   224 )      ! ETHP+HO2=OP2
     &                     +                 RXRAT(   263 )      ! ETHP+MO2=HO2+0.7500D+...
     &                     +                 RXRAT(   300 )      ! ETHP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   339 )      ! ETHP+NO3=HO2+NO2+ACD

      PROD(  ACT         ) =    3.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.3000D-01*ACT+...
     &                     +    1.1000D-01 * RXRAT(   144 )      ! OLI+O3=0.1100D+00*ACT+...
     &                     +    1.6500D-01 * RXRAT(   183 )      ! HC3P+NO=0.1650D+00*ACT+...
     &                     +    2.1700D-01 * RXRAT(   184 )      ! HC5P+NO=0.2170D+00*ACT+...
     &                     +    6.0000D-02 * RXRAT(   186 )      ! OLTP+NO=0.6000D-01*ACT+...
     &                     +    2.0000D-01 * RXRAT(   187 )      ! OLIP+NO=0.2000D+00*ACT+...
     &                     +    9.5000D-02 * RXRAT(   340 )      ! HC3P+NO3=0.9500D-01*ACT+...
     &                     +    2.4700D-01 * RXRAT(   341 )      ! HC5P+NO3=0.2470D+00*ACT+...
     &                     +    9.0000D-02 * RXRAT(   343 )      ! OLTP+NO3=0.9000D-01*ACT+...
     &                     +    2.0000D-01 * RXRAT(   344 )      ! OLIP+NO3=0.2000D+00*ACT+...
      LOSS(  ACT         ) =                 RXRAT(    14 )      ! ACT=MO2+ACO3
     &                     +                 RXRAT(    15 )      ! ACT=0.2000D+01*MO2+CO
     &                     +                 RXRAT(   110 )      ! ACT+HO=ACTP

      PROD(  UALD        ) =    7.0000D-01 * RXRAT(    23 )      ! MVK=0.7000D+00*UALD+...
     &                     +    2.8000D-01 * RXRAT(   196 )      ! LIMP1+NO=0.2800D+00*UALD+...
     &                     +    3.3300D-01 * RXRAT(   394 )      ! BDE13+HO=0.3330D+00*UALD+...
     &                     +    1.9000D-02 * RXRAT(   398 )      ! BDE13P+MO2=0.1900D-01*UALD+...
     &                     +    1.5000D-02 * RXRAT(   399 )      ! BDE13P+ACO3=0.1500D-...
      LOSS(  UALD        ) =                 RXRAT(    16 )      ! UALD=0.1220D+01*HO2+...
     &                     +                 RXRAT(   116 )      ! UALD+HO=0.3130D+00*ACO3+...
     &                     +                 RXRAT(   151 )      ! UALD+O3=0.1000D+00*HO+...
     &                     +                 RXRAT(   166 )      ! UALD+NO3=HO2+XO2+0.6680D+...

      PROD(  KET         ) =    2.1600D-01 * RXRAT(    16 )      ! UALD=0.2160D+00*KET+...
     &                     +    8.0000D-01 * RXRAT(    35 )      ! ONIT=0.8000D+00*KET+NO2+...
     &                     +    7.8000D-01 * RXRAT(   119 )      ! DCB1+HO=0.7800D+00*KET+...
     &                     +    4.1000D-01 * RXRAT(   131 )      ! OP2+HO=0.4100D+00*KET+...
     &                     +    4.1000D-01 * RXRAT(   132 )      ! OPB+HO=0.4100D+00*KET+...
     &                     +    1.7000D-02 * RXRAT(   144 )      ! OLI+O3=0.1700D-01*KET+...
     &                     +    3.3000D-01 * RXRAT(   147 )      ! LIMAL+O3=0.3300D+00*KET+...
     &                     +    2.8000D-02 * RXRAT(   151 )      ! UALD+O3=0.2800D-01*KET+...
     &                     +    3.3000D-02 * RXRAT(   184 )      ! HC5P+NO=0.3300D-01*KET+...
     &                     +    9.0000D-02 * RXRAT(   187 )      ! OLIP+NO=0.9000D-01*KET+...
     &                     +    9.5000D-01 * RXRAT(   200 )      ! PINALP+NO=0.9500D+00*KET+...
     &                     +    9.4000D-01 * RXRAT(   201 )      ! LIMALP+NO=0.9400D+00*KET+...
     &                     +    7.0000D-01 * RXRAT(   210 )      ! UALP+NO=0.7000D+00*KET+NO2+...
     &                     +    4.6400D-01 * RXRAT(   217 )      ! OLND+NO=0.4640D+00*KET+...
     &                     +    4.9700D-01 * RXRAT(   264 )      ! HC3P+MO2=0.4970D+00*KET+...
     &                     +    6.1800D-01 * RXRAT(   265 )      ! HC5P+MO2=0.6180D+00*KET+...
     &                     +    4.5000D-02 * RXRAT(   267 )      ! OLTP+MO2=0.4500D-01*KET+...
     &                     +    2.1800D-01 * RXRAT(   268 )      ! OLIP+MO2=0.2180D+00*KET+...
     &                     +    7.0000D-02 * RXRAT(   274 )      ! APIP1+MO2=0.7000D-01*KET+...
     &                     +    3.0000D-01 * RXRAT(   278 )      ! LIMP1+MO2=0.3000D+00*KET+...
     &                     +    5.2500D-01 * RXRAT(   290 )      ! UALP+MO2=0.5250D+00*KET+...
     &                     +    3.4800D-01 * RXRAT(   297 )      ! OLND+MO2=0.3480D+00*KET+...
     &                     +    6.6200D-01 * RXRAT(   301 )      ! HC3P+ACO3=0.6620D+00*KET+...
     &                     +    8.2400D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.8240D+00*KET+...
     &                     +    6.0000D-02 * RXRAT(   304 )      ! OLTP+ACO3=0.6000D-01*KET+...
     &                     +    2.9000D-01 * RXRAT(   305 )      ! OLIP+ACO3=0.2900D+00*KET+...
     &                     +    7.0000D-02 * RXRAT(   311 )      ! APIP1+ACO3=0.7000D-01*KET+...
     &                     +    3.0000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.3000D+00*KET+...
     &                     +    7.0000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.7000D+00*KET+...
     &                     +    4.6400D-01 * RXRAT(   334 )      ! OLND+ACO3=0.4640D+00*KET+...
     &                     +    6.0000D-02 * RXRAT(   341 )      ! HC5P+NO3=0.6000D-01*KET+...
     &                     +    1.1000D-01 * RXRAT(   344 )      ! OLIP+NO3=0.1100D+00*KET+...
     &                     +                 RXRAT(   350 )      ! APIP1+NO3=KET+NO2+ALD+HO2
     &                     +    7.0000D-01 * RXRAT(   360 )      ! UALP+NO3=0.7000D+00*KET+...
     &                     +    4.6400D-01 * RXRAT(   367 )      ! OLND+NO3=0.4640D+00*KET+...
     &                     +    1.4900D-01 * RXRAT(   370 )      ! OLNN+OLND=0.1490D+00*KET+...
     &                     +    2.8500D-01 * RXRAT(   371 )      ! OLND+OLND=0.2850D+00*KET+...
     &                     +    6.5000D-01 * RXRAT(   405 )      ! FURANONE+HO=0.6500D+00*KET+...
     &                     +    8.8000D-01 * RXRAT(   469 )      ! HC10P2+NO=0.8800D+00*KET+...
     &                     +                 RXRAT(   476 )      ! HC10P2+NO3=KET+NO2+HO2
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
      LOSS(  KET         ) =                 RXRAT(    20 )      ! KET=0.1500D+01*ETHP+...
     &                     +                 RXRAT(   112 )      ! KET+HO=KETP

      PROD(  PINAL       ) =    5.0000D-02 * RXRAT(   145 )      ! API+O3=0.5000D-01*PINAL+...
     &                     +    8.2000D-01 * RXRAT(   192 )      ! APIP1+NO=0.8200D+00*PINAL+...
     &                     +                 RXRAT(   194 )      ! APINP1+NO=PINAL+0.2000D+01*NO2
     &                     +    6.0000D-01 * RXRAT(   274 )      ! APIP1+MO2=0.6000D+00*PINAL+...
     &                     +    8.6000D-01 * RXRAT(   276 )      ! APINP1+MO2=0.8600D+...
     &                     +    6.0000D-01 * RXRAT(   311 )      ! APIP1+ACO3=0.6000D+...
     &                     +    8.6000D-01 * RXRAT(   313 )      ! APINP1+ACO3=0.8600D+...
     &                     +    4.8000D-01 * RXRAT(   375 )      ! APIP2+APIP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   378 )      ! LIMP2+APIP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   381 )      ! APINP2+APIP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   384 )      ! LIMNP2+APIP1=0.4800D+...
      LOSS(  PINAL       ) =                 RXRAT(    17 )      ! PINAL=HO2+HC10P+CO
     &                     +                 RXRAT(   105 )      ! PINAL+HO=0.2300D+00*PINALP+...

      PROD(  HC10P       ) =                 RXRAT(    17 )      ! PINAL=HC10P+HO2+CO
     &                     +                 RXRAT(    18 )      ! LIMAL=HC10P+HO2+CO
     &                     +    4.4000D-01 * RXRAT(   132 )      ! OPB+HO=0.4400D+00*HC10P+...
     &                     +    6.7000D-01 * RXRAT(   147 )      ! LIMAL+O3=0.6700D+00*HC10P+...
     &                     +                 RXRAT(   434 )      ! HC10+HO=HC10P
      LOSS(  HC10P       ) =                 RXRAT(   441 )      ! HC10P+NO=0.7400D+00*HC10P2+...
     &                     +                 RXRAT(   448 )      ! HC10P+NO3=HC10P2+NO2
     &                     +                 RXRAT(   455 )      ! HC10P+HO2=OP2

      PROD(  LIMAL       ) =    5.0000D-02 * RXRAT(   146 )      ! LIM+O3=0.5000D-01*LIMAL+...
     &                     +    4.9000D-01 * RXRAT(   196 )      ! LIMP1+NO=0.4900D+00*LIMAL+...
     &                     +                 RXRAT(   198 )      ! LIMNP1+NO=LIMAL+0.2000D+01*NO2
     &                     +    4.2000D-01 * RXRAT(   278 )      ! LIMP1+MO2=0.4200D+00*LIMAL+...
     &                     +    7.0000D-01 * RXRAT(   280 )      ! LIMNP1+MO2=0.7000D+...
     &                     +    4.2000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.4200D+...
     &                     +    7.0000D-01 * RXRAT(   317 )      ! LIMNP1+ACO3=0.7000D+...
     &                     +    4.8000D-01 * RXRAT(   376 )      ! APIP2+LIMP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   379 )      ! LIMP2+LIMP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   382 )      ! APINP2+LIMP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   385 )      ! LIMNP2+LIMP1=0.4800D+...
      LOSS(  LIMAL       ) =                 RXRAT(    18 )      ! LIMAL=HO2+HC10P+CO
     &                     +                 RXRAT(   106 )      ! LIMAL+HO=0.7000D+00*LIMALP+...
     &                     +                 RXRAT(   147 )      ! LIMAL+O3=0.4000D-01*HO+...

      PROD(  MEK         ) =    1.3000D-01 * RXRAT(   120 )      ! DCB2+HO=0.1300D+00*MEK+...
     &                     +    6.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.6000D-01*MEK+...
     &                     +    4.2000D-02 * RXRAT(   183 )      ! HC3P+NO=0.4200D-01*MEK+...
     &                     +    3.3000D-02 * RXRAT(   184 )      ! HC5P+NO=0.3300D-01*MEK+...
     &                     +    1.3000D-01 * RXRAT(   186 )      ! OLTP+NO=0.1300D+00*MEK+...
     &                     +    7.5000D-02 * RXRAT(   340 )      ! HC3P+NO3=0.7500D-01*MEK+...
     &                     +    6.3000D-02 * RXRAT(   341 )      ! HC5P+NO3=0.6300D-01*MEK+...
     &                     +    1.8000D-01 * RXRAT(   343 )      ! OLTP+NO3=0.1800D+00*MEK+...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*MEK+...
      LOSS(  MEK         ) =                 RXRAT(    19 )      ! MEK=0.1000D+00*MO2+ETHP+...
     &                     +                 RXRAT(   111 )      ! MEK+HO=MEKP

      PROD(  HKET        ) =                 RXRAT(   139 )      ! MPAN+HO=HKET+NO2
     &                     +    1.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.1000D-01*HKET+...
     &                     +    4.4000D-02 * RXRAT(   144 )      ! OLI+O3=0.4400D-01*HKET+...
     &                     +    2.7200D-01 * RXRAT(   184 )      ! HC5P+NO=0.2720D+00*HKET+...
     &                     +    2.0000D-02 * RXRAT(   187 )      ! OLIP+NO=0.2000D-01*HKET+...
     &                     +                 RXRAT(   208 )      ! MCP+NO=HKET+0.5000D+00*HO2+...
     &                     +    3.2000D-01 * RXRAT(   213 )      ! ADDC+NO=0.3200D+00*HKET+...
     &                     +    1.6000D-02 * RXRAT(   273 )      ! ISOP+MO2=0.1600D-01*HKET+...
     &                     +    5.0000D-01 * RXRAT(   288 )      ! MCP+MO2=0.5000D+00*HKET+...
     &                     +    3.2000D-01 * RXRAT(   293 )      ! ADDC+MO2=0.3200D+00*HKET+...
     &                     +    5.0000D-01 * RXRAT(   325 )      ! MCP+ACO3=0.5000D+00*HKET+...
     &                     +    3.2000D-01 * RXRAT(   330 )      ! ADDC+ACO3=0.3200D+00*HKET+...
     &                     +    2.7500D-01 * RXRAT(   341 )      ! HC5P+NO3=0.2750D+00*HKET+...
     &                     +    4.0000D-02 * RXRAT(   344 )      ! OLIP+NO3=0.4000D-01*HKET+...
     &                     +    3.3000D-02 * RXRAT(   349 )      ! ISOP+NO3=0.3300D-01*HKET+...
     &                     +                 RXRAT(   358 )      ! MCP+NO3=HKET+HO2+HCHO+NO2
     &                     +    3.2000D-01 * RXRAT(   363 )      ! ADDC+NO3=0.3200D+00*HKET+...
     &                     +    6.1300D-01 * RXRAT(   408 )      ! PROG+HO=0.6130D+00*HKET+...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
      LOSS(  HKET        ) =                 RXRAT(    21 )      ! HKET=HO2+ACO3+HCHO
     &                     +                 RXRAT(   113 )      ! HKET+HO=HO2+MGLY

      PROD(  MACR        ) =    5.3100D-01 * RXRAT(    87 )      ! ISO+O3=0.5310D+00*MACR+...
     &                     +    2.8000D-01 * RXRAT(    91 )      ! ISOP+NO=0.2800D+00*MACR+...
     &                     +    1.0000D-01 * RXRAT(    94 )      ! INO2+NO=0.1000D+00*MACR+...
     &                     +    4.0000D-01 * RXRAT(    95 )      ! IPN+HO2=0.4000D+00*MACR+...
     &                     +                 RXRAT(    98 )      ! ISHP+HO=MACR+0.1000D+...
     &                     +                 RXRAT(    99 )      ! ISHP=MACR+0.1000D+00*MGLY+...
     &                     +    1.5900D-01 * RXRAT(   273 )      ! ISOP+MO2=0.1590D+00*MACR+...
     &                     +    2.1900D-01 * RXRAT(   310 )      ! ISOP+ACO3=0.2190D+00*MACR+...
     &                     +    3.1800D-01 * RXRAT(   349 )      ! ISOP+NO3=0.3180D+00*MACR+...
     &                     +    6.1500D-01 * RXRAT(   351 )      ! LIMP1+NO3=0.6150D+00*MACR+...
     &                     +    9.0000D-01 * RXRAT(   401 )      ! BDE13+NO3=0.9000D+00*MACR+...
      LOSS(  MACR        ) =                 RXRAT(    22 )      ! MACR=0.3400D+00*HO+0.6600D+...
     &                     +                 RXRAT(   114 )      ! MACR+HO=0.5700D+00*MACP+...
     &                     +                 RXRAT(   149 )      ! MACR+O3=0.1900D+00*HO+...
     &                     +                 RXRAT(   165 )      ! MACR+NO3=0.6800D+00*HCHO+...

      PROD(  MACP        ) =    3.3000D-01 * RXRAT(    22 )      ! MACR=0.3300D+00*MACP+...
     &                     +    3.0000D-01 * RXRAT(    23 )      ! MVK=0.3000D+00*MACP+...
     &                     +    1.0000D-01 * RXRAT(    87 )      ! ISO+O3=0.1000D+00*MACP+...
     &                     +    5.7000D-01 * RXRAT(   114 )      ! MACR+HO=0.5700D+00*MACP+...
     &                     +    2.1000D-01 * RXRAT(   121 )      ! DCB3+HO=0.2100D+00*MACP+...
     &                     +                 RXRAT(   133 )      ! MAHP+HO=MACP
     &                     +    3.2000D-01 * RXRAT(   165 )      ! MACR+NO3=0.3200D+00*MACP+...
     &                     +                 RXRAT(   172 )      ! MPAN+NO3=MACP+NO2
     &                     +                 RXRAT(   180 )      ! MPAN=MACP+NO2
     &                     +    5.7000D-01 * RXRAT(   390 )      ! ACRO+HO=0.5700D+00*MACP+...
     &                     +    3.2000D-01 * RXRAT(   392 )      ! ACRO+NO3=0.3200D+00*MACP+...
      LOSS(  MACP        ) =                 RXRAT(   179 )      ! MACP+NO2=MPAN
     &                     +                 RXRAT(   207 )      ! MACP+NO=0.6500D+00*MO2+...
     &                     +                 RXRAT(   250 )      ! MACP+HO2=MAHP
     &                     +                 RXRAT(   287 )      ! MACP+MO2=0.5000D+00*HO2+...
     &                     +                 RXRAT(   324 )      ! MACP+ACO3=0.6350D+00*ORA2+...
     &                     +                 RXRAT(   357 )      ! MACP+NO3=HCHO+0.5380D+...

      PROD(  XO2         ) =    3.4000D-01 * RXRAT(    22 )      ! MACR=0.3400D+00*XO2+...
     &                     +    2.0000D-01 * RXRAT(    28 )      ! DCB1=0.2000D+00*XO2+...
     &                     +    2.0000D-01 * RXRAT(    29 )      ! DCB2=0.2000D+00*XO2+...
     &                     +    7.0000D-02 * RXRAT(   131 )      ! OP2+HO=0.7000D-01*XO2+...
     &                     +    7.0000D-02 * RXRAT(   132 )      ! OPB+HO=0.7000D-01*XO2+...
     &                     +    3.5000D-01 * RXRAT(   136 )      ! PAA+HO=0.3500D+00*XO2+...
     &                     +                 RXRAT(   137 )      ! PAN+HO=XO2+NO3+HCHO
     &                     +                 RXRAT(   138 )      ! PPN+HO=XO2+NO3+HCHO
     &                     +    1.0000D-02 * RXRAT(   150 )      ! MVK+O3=0.1000D-01*XO2+...
     &                     +    1.0000D-01 * RXRAT(   151 )      ! UALD+O3=0.1000D+00*XO2+...
     &                     +    6.0000D-01 * RXRAT(   152 )      ! DCB1+O3=0.6000D+00*XO2+HO2+...
     &                     +    6.0000D-01 * RXRAT(   153 )      ! DCB2+O3=0.6000D+00*XO2+HO2+...
     &                     +    6.8000D-01 * RXRAT(   165 )      ! MACR+NO3=0.6800D+00*XO2+...
     &                     +                 RXRAT(   166 )      ! UALD+NO3=XO2+HO2+0.6680D+...
     &                     +    8.9000D-02 * RXRAT(   183 )      ! HC3P+NO=0.8900D-01*XO2+...
     &                     +    2.3500D-01 * RXRAT(   184 )      ! HC5P+NO=0.2350D+00*XO2+...
     &                     +    1.6000D-01 * RXRAT(   206 )      ! KETP+NO=0.1600D+00*XO2+...
     &                     +    7.0000D-01 * RXRAT(   209 )      ! MVKP+NO=0.7000D+00*XO2+...
     &                     +    2.6000D-02 * RXRAT(   264 )      ! HC3P+MO2=0.2600D-01*XO2+...
     &                     +    1.9100D-01 * RXRAT(   265 )      ! HC5P+MO2=0.1910D+00*XO2+...
     &                     +    1.1600D+00 * RXRAT(   289 )      ! MVKP+MO2=0.1160D+01*XO2+...
     &                     +    2.6000D-02 * RXRAT(   301 )      ! HC3P+ACO3=0.2600D-01*XO2+...
     &                     +    1.9100D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.1910D+00*XO2+...
     &                     +    1.1600D+00 * RXRAT(   326 )      ! MVKP+ACO3=0.1160D+01*XO2+...
     &                     +    9.2000D-02 * RXRAT(   340 )      ! HC3P+NO3=0.9200D-01*XO2+...
     &                     +    4.8500D-01 * RXRAT(   341 )      ! HC5P+NO3=0.4850D+00*XO2+...
     &                     +    7.0000D-01 * RXRAT(   359 )      ! MVKP+NO3=0.7000D+00*XO2+...
     &                     +    6.8000D-01 * RXRAT(   392 )      ! ACRO+NO3=0.6800D+00*XO2+...
      LOSS(  XO2         ) =                 RXRAT(   219 )      ! XO2+NO=NO2
     &                     +                 RXRAT(   261 )      ! XO2+HO2=OP2
     &                     +                 RXRAT(   299 )      ! XO2+MO2=HO2+HCHO
     &                     +                 RXRAT(   336 )      ! XO2+ACO3=MO2
     &                     +                 RXRAT(   372 )      ! XO2+NO3=NO2
     &                     +                 RXRAT(   373 )      ! XO2+RCO3=ETHP
     &                     +    2.0000D+00 * RXRAT(   374 )      ! XO2+XO2=

      PROD(  MVK         ) =    1.8900D-01 * RXRAT(    87 )      ! ISO+O3=0.1890D+00*MVK+...
     &                     +    4.4000D-01 * RXRAT(    91 )      ! ISOP+NO=0.4400D+00*MVK+...
     &                     +    8.0000D-01 * RXRAT(    94 )      ! INO2+NO=0.8000D+00*MVK+...
     &                     +    9.4000D-01 * RXRAT(    95 )      ! IPN+HO2=0.9400D+00*MVK+...
     &                     +    2.0000D-01 * RXRAT(    98 )      ! ISHP+HO=0.2000D+00*MVK+...
     &                     +    2.5000D-01 * RXRAT(   273 )      ! ISOP+MO2=0.2500D+00*MVK+...
     &                     +    3.0500D-01 * RXRAT(   310 )      ! ISOP+ACO3=0.3050D+00*MVK+...
     &                     +    5.0000D-01 * RXRAT(   349 )      ! ISOP+NO3=0.5000D+00*MVK+...
     &                     +    4.8000D-01 * RXRAT(   377 )      ! APIP2+ISOP=0.4800D+00*MVK+...
     &                     +    4.8000D-01 * RXRAT(   380 )      ! LIMP2+ISOP=0.4800D+00*MVK+...
     &                     +    4.8000D-01 * RXRAT(   383 )      ! APINP2+ISOP=0.4800D+00*MVK+...
     &                     +    4.8000D-01 * RXRAT(   386 )      ! LIMNP2+ISOP=0.4800D+00*MVK+...
     &                     +    2.3100D-01 * RXRAT(   398 )      ! BDE13P+MO2=0.2310D+00*MVK+...
     &                     +    1.8500D-01 * RXRAT(   399 )      ! BDE13P+ACO3=0.1850D+00*MVK+...
      LOSS(  MVK         ) =                 RXRAT(    23 )      ! MVK=0.3000D+00*MO2+0.3000D+...
     &                     +                 RXRAT(   115 )      ! MVK+HO=MVKP
     &                     +                 RXRAT(   150 )      ! MVK+O3=0.1600D+00*HO+...

      PROD(  GLY         ) =    5.0000D-01 * RXRAT(    28 )      ! DCB1=0.5000D+00*GLY+...
     &                     +    5.0000D-01 * RXRAT(    29 )      ! DCB2=0.5000D+00*GLY+...
     &                     +    6.5000D-01 * RXRAT(    82 )      ! ACE+HO=0.6500D+00*GLY+...
     &                     +    2.1000D-02 * RXRAT(    91 )      ! ISOP+NO=0.2100D-01*GLY+...
     &                     +    6.0000D-01 * RXRAT(    95 )      ! IPN+HO2=0.6000D+00*GLY+...
     &                     +    2.0000D-01 * RXRAT(    98 )      ! ISHP+HO=0.2000D+00*GLY+...
     &                     +    1.0000D-01 * RXRAT(   119 )      ! DCB1+HO=0.1000D+00*GLY+...
     &                     +    1.0000D-01 * RXRAT(   120 )      ! DCB2+HO=0.1000D+00*GLY+...
     &                     +    2.7000D-01 * RXRAT(   121 )      ! DCB3+HO=0.2700D+00*GLY+...
     &                     +    4.9100D-01 * RXRAT(   151 )      ! UALD+O3=0.4910D+00*GLY+...
     &                     +    5.0000D-02 * RXRAT(   152 )      ! DCB1+O3=0.5000D-01*GLY+HO2+...
     &                     +    5.0000D-02 * RXRAT(   153 )      ! DCB2+O3=0.5000D-01*GLY+HO2+...
     &                     +    4.8000D-01 * RXRAT(   154 )      ! DCB3+O3=0.4800D+00*GLY+HO2+...
     &                     +    9.9800D-01 * RXRAT(   188 )      ! BENP+NO=0.9980D+00*GLY+...
     &                     +    5.4770D-01 * RXRAT(   189 )      ! TOLP+NO=0.5477D+00*GLY+...
     &                     +    7.0290D-01 * RXRAT(   190 )      ! XYMP+NO=0.7029D+00*GLY+...
     &                     +    5.4770D-01 * RXRAT(   191 )      ! XYEP+NO=0.5477D+00*GLY+...
     &                     +    1.8000D-01 * RXRAT(   210 )      ! UALP+NO=0.1800D+00*GLY+NO2+...
     &                     +    6.8000D-01 * RXRAT(   213 )      ! ADDC+NO=0.6800D+00*GLY+NO2+...
     &                     +                 RXRAT(   215 )      ! ORAP+NO=GLY+NO2+HO2
     &                     +                 RXRAT(   218 )      ! ADCN+NO=GLY+0.2000D+01*NO2+OP2
     &                     +    5.0000D-02 * RXRAT(   264 )      ! HC3P+MO2=0.5000D-01*GLY+...
     &                     +                 RXRAT(   269 )      ! BENP+MO2=GLY+0.1370D+...
     &                     +    5.4880D-01 * RXRAT(   270 )      ! TOLP+MO2=0.5488D+00*GLY+...
     &                     +    7.0430D-01 * RXRAT(   271 )      ! XYMP+MO2=0.7043D+00*GLY+...
     &                     +    5.4880D-01 * RXRAT(   272 )      ! XYEP+MO2=0.5488D+00*GLY+...
     &                     +    1.8000D-02 * RXRAT(   273 )      ! ISOP+MO2=0.1800D-01*GLY+...
     &                     +    1.3500D-01 * RXRAT(   290 )      ! UALP+MO2=0.1350D+00*GLY+...
     &                     +    6.8000D-01 * RXRAT(   293 )      ! ADDC+MO2=0.6800D+00*GLY+...
     &                     +                 RXRAT(   295 )      ! ORAP+MO2=GLY+HO2+HCHO
     &                     +    7.0000D-01 * RXRAT(   298 )      ! ADCN+MO2=0.7000D+00*GLY+...
     &                     +    6.7000D-02 * RXRAT(   301 )      ! HC3P+ACO3=0.6700D-01*GLY+...
     &                     +                 RXRAT(   306 )      ! BENP+ACO3=GLY+HO2+0.3000D+...
     &                     +    5.4880D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.5488D+00*GLY+...
     &                     +    7.0430D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.7043D+00*GLY+...
     &                     +    5.4880D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.5488D+00*GLY+...
     &                     +    1.8000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.1800D+00*GLY+...
     &                     +    6.8000D-01 * RXRAT(   330 )      ! ADDC+ACO3=0.6800D+00*GLY+...
     &                     +                 RXRAT(   332 )      ! ORAP+ACO3=GLY+MO2
     &                     +    7.0000D-01 * RXRAT(   335 )      ! ADCN+ACO3=0.7000D+00*GLY+...
     &                     +                 RXRAT(   345 )      ! BENP+NO3=GLY+HO2+0.0000D+...
     &                     +    5.4880D-01 * RXRAT(   346 )      ! TOLP+NO3=0.5488D+00*GLY+...
     &                     +    7.0430D-01 * RXRAT(   347 )      ! XYMP+NO3=0.7043D+00*GLY+...
     &                     +    5.4880D-01 * RXRAT(   348 )      ! XYEP+NO3=0.5488D+00*GLY+...
     &                     +    2.4000D-02 * RXRAT(   349 )      ! ISOP+NO3=0.2400D-01*GLY+...
     &                     +    1.8000D-01 * RXRAT(   360 )      ! UALP+NO3=0.1800D+00*GLY+...
     &                     +    6.8000D-01 * RXRAT(   363 )      ! ADDC+NO3=0.6800D+00*GLY+...
     &                     +                 RXRAT(   365 )      ! ORAP+NO3=GLY+NO2+HO2
     &                     +                 RXRAT(   368 )      ! ADCN+NO3=GLY+0.2000D+...
     &                     +    6.2000D-01 * RXRAT(   391 )      ! ACRO+O3=0.6200D+00*GLY+...
     &                     +    3.1000D-01 * RXRAT(   405 )      ! FURANONE+HO=0.3100D+00*GLY+...
     &                     +    4.6930D-01 * RXRAT(   486 )      ! VROCP6AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   487 )      ! VROCP6AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   492 )      ! VROCP5AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   493 )      ! VROCP5AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   498 )      ! NAPHP+NO=0.4693D+00*GLY+...
     &                     +    4.7020D-01 * RXRAT(   499 )      ! NAPHP+NO3=0.4702D+00*GLY+...
     &                     +    4.7020D-01 * RXRAT(   500 )      ! NAPHP+MO2=0.4702D+00*GLY+...
     &                     +    4.7020D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.4702D+00*GLY+...
      LOSS(  GLY         ) =                 RXRAT(    24 )      ! GLY=0.2000D+01*CO
     &                     +                 RXRAT(    25 )      ! GLY=HCHO+CO
     &                     +                 RXRAT(    26 )      ! GLY=0.2000D+01*HO2+0.2000D+...
     &                     +                 RXRAT(   117 )      ! GLY+HO=HO2+0.2000D+01*CO
     &                     +                 RXRAT(   167 )      ! GLY+NO3=HO2+0.2000D+01*CO+HNO3
     &                     +                 RXRAT(   418 )      ! GLY=AGLYJ

      PROD(  MGLY        ) =    5.0000D-01 * RXRAT(    28 )      ! DCB1=0.5000D+00*MGLY+...
     &                     +    5.0000D-01 * RXRAT(    29 )      ! DCB2=0.5000D+00*MGLY+...
     &                     +    5.0000D-01 * RXRAT(    94 )      ! INO2+NO=0.5000D+00*MGLY+...
     &                     +    9.0000D-01 * RXRAT(    95 )      ! IPN+HO2=0.9000D+00*MGLY+...
     &                     +    1.2000D-01 * RXRAT(    97 )      ! IHN+NO=0.1200D+00*MGLY+...
     &                     +    1.0000D-01 * RXRAT(    98 )      ! ISHP+HO=0.1000D+00*MGLY+...
     &                     +    1.0000D-01 * RXRAT(    99 )      ! ISHP=0.1000D+00*MGLY+...
     &                     +                 RXRAT(   113 )      ! HKET+HO=MGLY+HO2
     &                     +    1.0000D-02 * RXRAT(   119 )      ! DCB1+HO=0.1000D-01*MGLY+...
     &                     +    1.0000D-02 * RXRAT(   120 )      ! DCB2+HO=0.1000D-01*MGLY+...
     &                     +    1.0000D-02 * RXRAT(   121 )      ! DCB3+HO=0.1000D-01*MGLY+...
     &                     +    5.0000D-01 * RXRAT(   149 )      ! MACR+O3=0.5000D+00*MGLY+...
     &                     +    5.4000D-01 * RXRAT(   150 )      ! MVK+O3=0.5400D+00*MGLY+...
     &                     +    3.0000D-03 * RXRAT(   151 )      ! UALD+O3=0.3000D-02*MGLY+...
     &                     +    8.0000D-02 * RXRAT(   152 )      ! DCB1+O3=0.8000D-01*MGLY+...
     &                     +    8.0000D-02 * RXRAT(   153 )      ! DCB2+O3=0.8000D-01*MGLY+...
     &                     +    6.8000D-01 * RXRAT(   165 )      ! MACR+NO3=0.6800D+00*MGLY+...
     &                     +    3.6510D-01 * RXRAT(   189 )      ! TOLP+NO=0.3651D+00*MGLY+...
     &                     +    2.4700D-01 * RXRAT(   190 )      ! XYMP+NO=0.2470D+00*MGLY+...
     &                     +    3.6510D-01 * RXRAT(   191 )      ! XYEP+NO=0.3651D+00*MGLY+...
     &                     +    5.4000D-01 * RXRAT(   206 )      ! KETP+NO=0.5400D+00*MGLY+...
     &                     +    3.0000D-01 * RXRAT(   209 )      ! MVKP+NO=0.3000D+00*MGLY+...
     &                     +    2.1000D-01 * RXRAT(   210 )      ! UALP+NO=0.2100D+00*MGLY+...
     &                     +    3.6590D-01 * RXRAT(   270 )      ! TOLP+MO2=0.3659D+00*MGLY+...
     &                     +    2.4750D-01 * RXRAT(   271 )      ! XYMP+MO2=0.2475D+00*MGLY+...
     &                     +    3.6590D-01 * RXRAT(   272 )      ! XYEP+MO2=0.3659D+00*MGLY+...
     &                     +    5.0000D-01 * RXRAT(   289 )      ! MVKP+MO2=0.5000D+00*MGLY+...
     &                     +    1.0500D-01 * RXRAT(   290 )      ! UALP+MO2=0.1050D+00*MGLY+...
     &                     +    3.6590D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.3659D+00*MGLY+...
     &                     +    2.4750D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.2475D+00*MGLY+...
     &                     +    3.6590D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.3659D+00*MGLY+...
     &                     +    5.0000D-01 * RXRAT(   326 )      ! MVKP+ACO3=0.5000D+00*MGLY+...
     &                     +    1.0500D-01 * RXRAT(   327 )      ! UALP+ACO3=0.1050D+00*MGLY+...
     &                     +    3.6590D-01 * RXRAT(   346 )      ! TOLP+NO3=0.3659D+00*MGLY+...
     &                     +    2.4750D-01 * RXRAT(   347 )      ! XYMP+NO3=0.2475D+00*MGLY+...
     &                     +    3.6590D-01 * RXRAT(   348 )      ! XYEP+NO3=0.3659D+00*MGLY+...
     &                     +    3.0000D-01 * RXRAT(   359 )      ! MVKP+NO3=0.3000D+00*MGLY+...
     &                     +    2.1000D-01 * RXRAT(   360 )      ! UALP+NO3=0.2100D+00*MGLY+...
     &                     +    6.8000D-01 * RXRAT(   392 )      ! ACRO+NO3=0.6800D+00*MGLY+...
     &                     +    4.6930D-01 * RXRAT(   486 )      ! VROCP6AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   487 )      ! VROCP6AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   492 )      ! VROCP5AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   493 )      ! VROCP5AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   498 )      ! NAPHP+NO=0.4693D+00*MGLY+...
     &                     +    4.7020D-01 * RXRAT(   499 )      ! NAPHP+NO3=0.4702D+00*MGLY+...
     &                     +    4.7020D-01 * RXRAT(   500 )      ! NAPHP+MO2=0.4702D+00*MGLY+...
     &                     +    4.7020D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.4702D+00*MGLY+...
      LOSS(  MGLY        ) =                 RXRAT(    27 )      ! MGLY=HO2+ACO3+CO
     &                     +                 RXRAT(   118 )      ! MGLY+HO=ACO3+CO
     &                     +                 RXRAT(   168 )      ! MGLY+NO3=ACO3+CO+HNO3
     &                     +                 RXRAT(   419 )      ! MGLY=AGLYJ

      PROD(  DCB1        ) =    7.0000D-01 * RXRAT(   153 )      ! DCB2+O3=0.7000D+00*DCB1+...
     &                     +    7.0000D-01 * RXRAT(   154 )      ! DCB3+O3=0.7000D+00*DCB1+...
     &                     +    5.4770D-01 * RXRAT(   189 )      ! TOLP+NO=0.5477D+00*DCB1+...
     &                     +    6.7000D-01 * RXRAT(   205 )      ! MEKP+NO=0.6700D+00*DCB1+...
     &                     +    5.4880D-01 * RXRAT(   270 )      ! TOLP+MO2=0.5488D+00*DCB1+...
     &                     +    3.3400D-01 * RXRAT(   285 )      ! MEKP+MO2=0.3340D+00*DCB1+...
     &                     +    5.0000D-01 * RXRAT(   286 )      ! KETP+MO2=0.5000D+00*DCB1+...
     &                     +    5.4880D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.5488D+00*DCB1+...
     &                     +    3.3400D-01 * RXRAT(   322 )      ! MEKP+ACO3=0.3340D+00*DCB1+...
     &                     +    5.0000D-01 * RXRAT(   323 )      ! KETP+ACO3=0.5000D+00*DCB1+...
     &                     +    5.4880D-01 * RXRAT(   346 )      ! TOLP+NO3=0.5488D+00*DCB1+...
     &                     +    6.7000D-01 * RXRAT(   355 )      ! MEKP+NO3=0.6700D+00*DCB1+...
     &                     +                 RXRAT(   356 )      ! KETP+NO3=DCB1+NO2+HO2
     &                     +    4.9000D-01 * RXRAT(   402 )      ! FURAN+HO=0.4900D+00*DCB1+...
     &                     +    8.0000D-01 * RXRAT(   407 )      ! FURAN+NO3=0.8000D+00*DCB1+...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*DCB1+...
      LOSS(  DCB1        ) =                 RXRAT(    28 )      ! DCB1=0.1500D+01*HO2+...
     &                     +                 RXRAT(   119 )      ! DCB1+HO=0.5200D+00*HO2+...
     &                     +                 RXRAT(   152 )      ! DCB1+O3=0.5000D-01*HO+HO2+...

      PROD(  DCB2        ) =    2.4950D-01 * RXRAT(   188 )      ! BENP+NO=0.2495D+00*DCB2+...
     &                     +    5.9840D-01 * RXRAT(   190 )      ! XYMP+NO=0.5984D+00*DCB2+...
     &                     +    4.5640D-01 * RXRAT(   191 )      ! XYEP+NO=0.4564D+00*DCB2+...
     &                     +    2.5000D-01 * RXRAT(   269 )      ! BENP+MO2=0.2500D+00*DCB2+...
     &                     +    5.9960D-01 * RXRAT(   271 )      ! XYMP+MO2=0.5996D+00*DCB2+...
     &                     +    4.5730D-01 * RXRAT(   272 )      ! XYEP+MO2=0.4573D+00*DCB2+...
     &                     +    2.5000D-01 * RXRAT(   306 )      ! BENP+ACO3=0.2500D+00*DCB2+...
     &                     +    5.9960D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.5996D+00*DCB2+...
     &                     +    4.5730D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.4573D+00*DCB2+...
     &                     +    2.5000D-01 * RXRAT(   345 )      ! BENP+NO3=0.2500D+00*DCB2+...
     &                     +    5.9960D-01 * RXRAT(   347 )      ! XYMP+NO3=0.5996D+00*DCB2+...
     &                     +    4.5730D-01 * RXRAT(   348 )      ! XYEP+NO3=0.4573D+00*DCB2+...
     &                     +    4.6930D-01 * RXRAT(   486 )      ! VROCP6AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   487 )      ! VROCP6AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   492 )      ! VROCP5AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   493 )      ! VROCP5AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   498 )      ! NAPHP+NO=0.4693D+00*DCB2+...
     &                     +    4.7020D-01 * RXRAT(   499 )      ! NAPHP+NO3=0.4702D+00*DCB2+...
     &                     +    4.7020D-01 * RXRAT(   500 )      ! NAPHP+MO2=0.4702D+00*DCB2+...
     &                     +    4.7020D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.4702D+00*DCB2+...
      LOSS(  DCB2        ) =                 RXRAT(    29 )      ! DCB2=0.1500D+01*HO2+...
     &                     +                 RXRAT(   120 )      ! DCB2+HO=0.5200D+00*HO2+...
     &                     +                 RXRAT(   153 )      ! DCB2+O3=0.5000D-01*HO+HO2+...

      PROD(  BALD        ) =    2.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.2000D-01*BALD+...
     &                     +    8.5200D-02 * RXRAT(   189 )      ! TOLP+NO=0.8520D-01*BALD+...
     &                     +    4.8100D-02 * RXRAT(   190 )      ! XYMP+NO=0.4810D-01*BALD+...
     &                     +    8.5200D-02 * RXRAT(   191 )      ! XYEP+NO=0.8520D-01*BALD+...
     &                     +    8.5400D-02 * RXRAT(   270 )      ! TOLP+MO2=0.8540D-01*BALD+...
     &                     +    4.8200D-02 * RXRAT(   271 )      ! XYMP+MO2=0.4820D-01*BALD+...
     &                     +    8.5400D-02 * RXRAT(   272 )      ! XYEP+MO2=0.8540D-01*BALD+...
     &                     +    8.5400D-02 * RXRAT(   307 )      ! TOLP+ACO3=0.8540D-01*BALD+...
     &                     +    4.8200D-02 * RXRAT(   308 )      ! XYMP+ACO3=0.4820D-01*BALD+...
     &                     +    8.5400D-02 * RXRAT(   309 )      ! XYEP+ACO3=0.8540D-01*BALD+...
     &                     +    8.5400D-02 * RXRAT(   346 )      ! TOLP+NO3=0.8540D-01*BALD+...
     &                     +    4.8200D-02 * RXRAT(   347 )      ! XYMP+NO3=0.4820D-01*BALD+...
     &                     +    8.5400D-02 * RXRAT(   348 )      ! XYEP+NO3=0.8540D-01*BALD+...
     &                     +    5.9400D-02 * RXRAT(   486 )      ! VROCP6AROP+NO=0.5940D-...
     &                     +    5.9500D-02 * RXRAT(   487 )      ! VROCP6AROP+NO3=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.5950D-...
      LOSS(  BALD        ) =                 RXRAT(    30 )      ! BALD=CHO+HO2+CO
     &                     +                 RXRAT(   122 )      ! BALD+HO=BALP

      PROD(  CHO         ) =                 RXRAT(    30 )      ! BALD=CHO+HO2+CO
     &                     +    5.9000D-02 * RXRAT(   123 )      ! PHEN+HO=0.5900D-01*CHO+...
     &                     +    5.6000D-02 * RXRAT(   124 )      ! CSL+HO=0.5600D-01*CHO+...
     &                     +    3.3900D-01 * RXRAT(   169 )      ! PHEN+NO3=0.3390D+00*CHO+...
     &                     +    3.2000D-01 * RXRAT(   170 )      ! CSL+NO3=0.3200D+00*CHO+...
      LOSS(  CHO         ) =                 RXRAT(   221 )      ! CHO+NO2=ONIT
     &                     +                 RXRAT(   255 )      ! CHO+HO2=CSL

      PROD(  OP1         ) =                 RXRAT(   223 )      ! MO2+HO2=OP1
      LOSS(  OP1         ) =                 RXRAT(    31 )      ! OP1=HO+HO2+HCHO
     &                     +                 RXRAT(   130 )      ! OP1+HO=0.3500D+00*HO+...

      PROD(  OP2         ) =    7.8000D-01 * RXRAT(   120 )      ! DCB2+HO=0.7800D+00*OP2+...
     &                     +    7.9000D-01 * RXRAT(   121 )      ! DCB3+HO=0.7900D+00*OP2+...
     &                     +    6.5000D-01 * RXRAT(   152 )      ! DCB1+O3=0.6500D+00*OP2+HO2+...
     &                     +    6.5000D-01 * RXRAT(   153 )      ! DCB2+O3=0.6500D+00*OP2+HO2+...
     &                     +    6.8000D-01 * RXRAT(   213 )      ! ADDC+NO=0.6800D+00*OP2+NO2+...
     &                     +                 RXRAT(   218 )      ! ADCN+NO=OP2+GLY+0.2000D+01*NO2
     &                     +                 RXRAT(   224 )      ! ETHP+HO2=OP2
     &                     +                 RXRAT(   225 )      ! HC3P+HO2=OP2
     &                     +                 RXRAT(   226 )      ! HC5P+HO2=OP2
     &                     +                 RXRAT(   227 )      ! ETEP+HO2=OP2
     &                     +                 RXRAT(   228 )      ! OLTP+HO2=OP2
     &                     +                 RXRAT(   229 )      ! OLIP+HO2=OP2
     &                     +    6.0210D-01 * RXRAT(   230 )      ! BENP+HO2=0.6021D+00*OP2+...
     &                     +    7.1950D-01 * RXRAT(   231 )      ! TOLP+HO2=0.7195D+00*OP2+...
     &                     +    4.8200D-02 * RXRAT(   232 )      ! XYMP+HO2=0.4820D-01*OP2+...
     &                     +    8.5400D-02 * RXRAT(   233 )      ! XYEP+HO2=0.8540D-01*OP2+...
     &                     +    8.5000D-01 * RXRAT(   247 )      ! ACTP+HO2=0.8500D+00*OP2+...
     &                     +                 RXRAT(   248 )      ! MEKP+HO2=OP2
     &                     +                 RXRAT(   249 )      ! KETP+HO2=OP2
     &                     +                 RXRAT(   252 )      ! MVKP+HO2=OP2
     &                     +                 RXRAT(   253 )      ! UALP+HO2=OP2
     &                     +                 RXRAT(   254 )      ! ADDC+HO2=OP2
     &                     +                 RXRAT(   256 )      ! MCTP+HO2=OP2
     &                     +                 RXRAT(   257 )      ! ORAP+HO2=OP2
     &                     +                 RXRAT(   260 )      ! ADCN+HO2=OP2
     &                     +                 RXRAT(   261 )      ! XO2+HO2=OP2
     &                     +    6.8000D-01 * RXRAT(   293 )      ! ADDC+MO2=0.6800D+00*OP2+...
     &                     +    7.0000D-01 * RXRAT(   298 )      ! ADCN+MO2=0.7000D+00*OP2+...
     &                     +    6.8000D-01 * RXRAT(   330 )      ! ADDC+ACO3=0.6800D+00*OP2+...
     &                     +    7.0000D-01 * RXRAT(   335 )      ! ADCN+ACO3=0.7000D+00*OP2+...
     &                     +    6.8000D-01 * RXRAT(   363 )      ! ADDC+NO3=0.6800D+00*OP2+...
     &                     +                 RXRAT(   368 )      ! ADCN+NO3=OP2+GLY+0.2000D+...
     &                     +                 RXRAT(   397 )      ! BDE13P+HO2=OP2
     &                     +    6.0000D-01 * RXRAT(   404 )      ! FURANO2+HO2=0.6000D+00*OP2+...
     &                     +                 RXRAT(   455 )      ! HC10P+HO2=OP2
     &                     +    4.0000D-03 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.4000D-...
     &                     +    1.3500D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.1350D-...
     &                     +    6.2000D-03 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.6200D-...
     &                     +    5.4000D-03 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.5400D-...
     &                     +    1.7000D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.1700D-...
     &                     +    2.5000D-03 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2500D-...
     &                     +    2.3000D-03 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.2300D-...
     &                     +    1.1400D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.1140D-...
     &                     +    1.5500D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.1550D-...
     &                     +    3.8400D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.3840D-...
     &                     +    5.8500D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5850D-...
      LOSS(  OP2         ) =                 RXRAT(    32 )      ! OP2=HO+HO2+ALD
     &                     +                 RXRAT(   131 )      ! OP2+HO=0.1000D-01*HO+...

      PROD(  OPB         ) =                 RXRAT(   235 )      ! APIP1+HO2=OPB
     &                     +                 RXRAT(   239 )      ! LIMP1+HO2=OPB
     &                     +                 RXRAT(   243 )      ! PINALP+HO2=OPB
     &                     +                 RXRAT(   244 )      ! LIMALP+HO2=OPB
      LOSS(  OPB         ) =                 RXRAT(    33 )      ! OPB=HO+HO2+ALD
     &                     +                 RXRAT(   132 )      ! OPB+HO=0.1000D-01*HO+...

      PROD(  PAA         ) =    1.1000D-01 * RXRAT(   154 )      ! DCB3+O3=0.1100D+00*PAA+HO2+...
     &                     +    4.1000D-01 * RXRAT(   245 )      ! ACO3+HO2=0.4100D+00*PAA+...
     &                     +    4.1000D-01 * RXRAT(   246 )      ! RCO3+HO2=0.4100D+00*PAA+...
      LOSS(  PAA         ) =                 RXRAT(    34 )      ! PAA=HO+MO2
     &                     +                 RXRAT(   136 )      ! PAA+HO=0.3500D+00*HO+...

      PROD(  ONIT        ) =                 RXRAT(   166 )      ! UALD+NO3=ONIT+XO2+0.6680D+...
     &                     +    6.5000D-02 * RXRAT(   183 )      ! HC3P+NO=0.6500D-01*ONIT+...
     &                     +    1.3600D-01 * RXRAT(   184 )      ! HC5P+NO=0.1360D+00*ONIT+...
     &                     +    3.0000D-02 * RXRAT(   186 )      ! OLTP+NO=0.3000D-01*ONIT+...
     &                     +    5.0000D-02 * RXRAT(   187 )      ! OLIP+NO=0.5000D-01*ONIT+...
     &                     +    2.0000D-04 * RXRAT(   189 )      ! TOLP+NO=0.2000D-03*ONIT+...
     &                     +    1.0000D-04 * RXRAT(   190 )      ! XYMP+NO=0.1000D-03*ONIT+...
     &                     +    2.0000D-04 * RXRAT(   191 )      ! XYEP+NO=0.2000D-03*ONIT+...
     &                     +                 RXRAT(   216 )      ! OLNN+NO=ONIT+HO2+NO2
     &                     +                 RXRAT(   220 )      ! BAL2+NO2=ONIT
     &                     +                 RXRAT(   221 )      ! CHO+NO2=ONIT
     &                     +                 RXRAT(   222 )      ! MCTO+NO2=ONIT
     &                     +                 RXRAT(   258 )      ! OLNN+HO2=ONIT
     &                     +                 RXRAT(   259 )      ! OLND+HO2=ONIT
     &                     +                 RXRAT(   296 )      ! OLNN+MO2=ONIT+HCHO+0.2000D+...
     &                     +    5.0000D-01 * RXRAT(   297 )      ! OLND+MO2=0.5000D+00*ONIT+...
     &                     +    3.0000D-01 * RXRAT(   298 )      ! ADCN+MO2=0.3000D+00*ONIT+...
     &                     +                 RXRAT(   333 )      ! OLNN+ACO3=ONIT+MO2+HO2
     &                     +    3.0000D-01 * RXRAT(   335 )      ! ADCN+ACO3=0.3000D+00*ONIT+...
     &                     +                 RXRAT(   366 )      ! OLNN+NO3=ONIT+NO2+HO2
     &                     +    2.0000D+00 * RXRAT(   369 )      ! OLNN+OLNN=0.2000D+01*ONIT+HO2
     &                     +    1.5000D+00 * RXRAT(   370 )      ! OLNN+OLND=0.1500D+01*ONIT+...
     &                     +                 RXRAT(   371 )      ! OLND+OLND=ONIT+0.5040D+...
     &                     +    3.2000D-02 * RXRAT(   395 )      ! BDE13P+NO=0.3200D-01*ONIT+...
     &                     +    8.0000D-02 * RXRAT(   403 )      ! FURANO2+NO=0.8000D-01*ONIT+...
     &                     +    2.6000D-01 * RXRAT(   441 )      ! HC10P+NO=0.2600D+00*ONIT+...
     &                     +    1.2000D-01 * RXRAT(   469 )      ! HC10P2+NO=0.1200D+00*ONIT+...
      LOSS(  ONIT        ) =                 RXRAT(    35 )      ! ONIT=HO2+NO2+0.2000D+...
     &                     +                 RXRAT(   140 )      ! ONIT+HO=HC3P+NO2

      PROD(  SO2         ) =   0.0D0
      LOSS(  SO2         ) =                 RXRAT(    73 )      ! SO2+HO=HO2+SULF+SULRXN

      PROD(  SULF        ) =                 RXRAT(    73 )      ! SO2+HO=SULF+HO2+SULRXN
      LOSS(  SULF        ) =   0.0D0

      PROD(  SULRXN      ) =                 RXRAT(    73 )      ! SO2+HO=SULRXN+SULF+HO2
      LOSS(  SULRXN      ) =   0.0D0

      PROD(  ETH         ) =    2.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.2000D-01*ETH+...
     &                     +    1.0000D-02 * RXRAT(   144 )      ! OLI+O3=0.1000D-01*ETH+...
      LOSS(  ETH         ) =                 RXRAT(    76 )      ! ETH+HO=ETHP

      PROD(  HC3         ) =    1.5000D-02 * RXRAT(   143 )      ! OLT+O3=0.1500D-01*HC3+...
     &                     +    1.0000D-02 * RXRAT(   144 )      ! OLI+O3=0.1000D-01*HC3+...
      LOSS(  HC3         ) =                 RXRAT(    77 )      ! HC3+HO=HC3P+0.3310D-04*ASOATJ

      PROD(  HC3P        ) =                 RXRAT(    77 )      ! HC3+HO=HC3P+0.3310D-04*ASOATJ
     &                     +    4.4000D-01 * RXRAT(   131 )      ! OP2+HO=0.4400D+00*HC3P+...
     &                     +                 RXRAT(   140 )      ! ONIT+HO=HC3P+NO2
     &                     +    4.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.4000D-01*HC3P+...
     &                     +    4.0000D-02 * RXRAT(   144 )      ! OLI+O3=0.4000D-01*HC3P+...
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*HC3P+...
      LOSS(  HC3P        ) =                 RXRAT(   183 )      ! HC3P+NO=0.6600D+00*HO2+...
     &                     +                 RXRAT(   225 )      ! HC3P+HO2=OP2
     &                     +                 RXRAT(   264 )      ! HC3P+MO2=0.8940D+00*HO2+...
     &                     +                 RXRAT(   301 )      ! HC3P+ACO3=0.3940D+00*HO2+...
     &                     +                 RXRAT(   340 )      ! HC3P+NO3=0.2540D+00*HO2+...

      PROD(  ASOATJ      ) =    3.3100D-05 * RXRAT(    77 )      ! HC3+HO=0.3310D-04*ASOATJ+HC3P
     &                     +    1.5800D-03 * RXRAT(    78 )      ! HC5+HO=0.1580D-02*ASOATJ+HC5P
     &                     +    1.5200D-01 * RXRAT(   123 )      ! PHEN+HO=0.1520D+00*ASOATJ+...
     &                     +    2.0000D-01 * RXRAT(   124 )      ! CSL+HO=0.2000D+00*ASOATJ+...
     &                     +    1.5200D-01 * RXRAT(   169 )      ! PHEN+NO3=0.1520D+00*ASOATJ+...
     &                     +    2.0000D-01 * RXRAT(   170 )      ! CSL+NO3=0.2000D+00*ASOATJ+...
     &                     +    1.4900D-01 * RXRAT(   388 )      ! VROCIOXY+HO=0.1490D+...
     &                     +    1.0100D-03 * RXRAT(   389 )      ! SLOWROC+HO=0.1010D-...
     &                     +    4.0000D-02 * RXRAT(   405 )      ! FURANONE+HO=0.4000D-...
      LOSS(  ASOATJ      ) =   0.0D0

      PROD(  HC5         ) =    6.0000D-03 * RXRAT(   143 )      ! OLT+O3=0.6000D-02*HC5+...
      LOSS(  HC5         ) =                 RXRAT(    78 )      ! HC5+HO=HC5P+0.1580D-02*ASOATJ

      PROD(  HC5P        ) =                 RXRAT(    78 )      ! HC5+HO=HC5P+0.1580D-02*ASOATJ
     &                     +    2.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.2000D-01*HC5P+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    9.2500D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.9250D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    1.7530D-01 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.1753D+...
     &                     +    5.3700D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.5370D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    1.0550D-01 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.1055D+...
     &                     +    2.2200D-01 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.2220D+...
     &                     +    6.5400D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.6540D-...
     &                     +    1.4890D-01 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.1489D+...
     &                     +    1.6560D-01 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.1656D+...
     &                     +    1.7300D-01 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.1730D+...
     &                     +    1.2800D-01 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1280D+...
     &                     +    1.5440D-01 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.1544D+...
      LOSS(  HC5P        ) =                 RXRAT(   184 )      ! HC5P+NO=0.2000D+00*HO2+...
     &                     +                 RXRAT(   226 )      ! HC5P+HO2=OP2
     &                     +                 RXRAT(   265 )      ! HC5P+MO2=0.8420D+00*HO2+...
     &                     +                 RXRAT(   302 )      ! HC5P+ACO3=0.3420D+00*HO2+...
     &                     +                 RXRAT(   341 )      ! HC5P+NO3=0.4880D+00*HO2+...

      PROD(  ETE         ) =    2.5000D-01 * RXRAT(   393 )      ! ACRO=0.2500D+00*ETE+...
     &                     +    1.7000D-01 * RXRAT(   400 )      ! BDE13+O3=0.1700D+00*ETE+...
      LOSS(  ETE         ) =                 RXRAT(    79 )      ! ETE+HO=ETEP
     &                     +                 RXRAT(   142 )      ! ETE+O3=0.8000D-01*HO+...
     &                     +                 RXRAT(   156 )      ! ETE+NO3=0.8000D+00*OLNN+...

      PROD(  ETEP        ) =                 RXRAT(    79 )      ! ETE+HO=ETEP
      LOSS(  ETEP        ) =                 RXRAT(   185 )      ! ETEP+NO=HO2+NO2+0.1600D+...
     &                     +                 RXRAT(   227 )      ! ETEP+HO2=OP2
     &                     +                 RXRAT(   266 )      ! ETEP+MO2=HO2+0.1950D+...
     &                     +                 RXRAT(   303 )      ! ETEP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   342 )      ! ETEP+NO3=HO2+NO2+0.1600D+...

      PROD(  OLT         ) =    1.5300D-01 * RXRAT(    87 )      ! ISO+O3=0.1530D+00*OLT+...
     &                     +    2.5000D-01 * RXRAT(   398 )      ! BDE13P+MO2=0.2500D+00*OLT+...
      LOSS(  OLT         ) =                 RXRAT(    80 )      ! OLT+HO=OLTP
     &                     +                 RXRAT(   143 )      ! OLT+O3=0.2200D+00*HO+...
     &                     +                 RXRAT(   157 )      ! OLT+NO3=0.4300D+00*OLNN+...

      PROD(  OLTP        ) =                 RXRAT(    80 )      ! OLT+HO=OLTP
      LOSS(  OLTP        ) =                 RXRAT(   186 )      ! OLTP+NO=0.7800D+00*HO2+...
     &                     +                 RXRAT(   228 )      ! OLTP+HO2=OP2
     &                     +                 RXRAT(   267 )      ! OLTP+MO2=HO2+0.1500D+...
     &                     +                 RXRAT(   304 )      ! OLTP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   343 )      ! OLTP+NO3=0.4700D+00*ALD+...

      PROD(  OLI         ) =    3.8500D-01 * RXRAT(   351 )      ! LIMP1+NO3=0.3850D+00*OLI+...
      LOSS(  OLI         ) =                 RXRAT(    81 )      ! OLI+HO=OLIP
     &                     +                 RXRAT(   144 )      ! OLI+O3=0.4600D+00*HO+...
     &                     +                 RXRAT(   158 )      ! OLI+NO3=0.1100D+00*OLNN+...

      PROD(  OLIP        ) =                 RXRAT(    81 )      ! OLI+HO=OLIP
      LOSS(  OLIP        ) =                 RXRAT(   187 )      ! OLIP+NO=0.8300D+00*HO2+...
     &                     +                 RXRAT(   229 )      ! OLIP+HO2=OP2
     &                     +                 RXRAT(   268 )      ! OLIP+MO2=HO2+0.7500D+...
     &                     +                 RXRAT(   305 )      ! OLIP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   344 )      ! OLIP+NO3=0.8600D+00*HO2+...

      PROD(  ACE         ) =   0.0D0
      LOSS(  ACE         ) =                 RXRAT(    82 )      ! ACE+HO=0.6500D+00*HO+...

      PROD(  ORA1        ) =    3.5000D-01 * RXRAT(    82 )      ! ACE+HO=0.3500D+00*ORA1+...
     &                     +    2.8000D-01 * RXRAT(    87 )      ! ISO+O3=0.2800D+00*ORA1+...
     &                     +    3.7000D-01 * RXRAT(   142 )      ! ETE+O3=0.3700D+00*ORA1+...
     &                     +    3.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.3000D-01*ORA1+...
     &                     +    4.5000D-01 * RXRAT(   149 )      ! MACR+O3=0.4500D+00*ORA1+...
     &                     +    7.0000D-02 * RXRAT(   150 )      ! MVK+O3=0.7000D-01*ORA1+...
     &                     +    4.4000D-02 * RXRAT(   151 )      ! UALD+O3=0.4400D-01*ORA1+...
     &                     +    2.5000D-01 * RXRAT(   154 )      ! DCB3+O3=0.2500D+00*ORA1+...
      LOSS(  ORA1        ) =                 RXRAT(   134 )      ! ORA1+HO=HO2

      PROD(  BEN         ) =    3.2000D-02 * RXRAT(   143 )      ! OLT+O3=0.3200D-01*BEN+...
      LOSS(  BEN         ) =                 RXRAT(    83 )      ! BEN+HO=0.4700D+00*BENP+...

      PROD(  BENP        ) =    4.7000D-01 * RXRAT(    83 )      ! BEN+HO=0.4700D+00*BENP+...
      LOSS(  BENP        ) =                 RXRAT(   188 )      ! BENP+NO=0.0000D+00*ONIT+...
     &                     +                 RXRAT(   230 )      ! BENP+HO2=0.6021D+00*OP2+...
     &                     +                 RXRAT(   269 )      ! BENP+MO2=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   306 )      ! BENP+ACO3=0.7000D+00*MO2+...
     &                     +                 RXRAT(   345 )      ! BENP+NO3=NO2+HO2+0.0000D+...

      PROD(  PHEN        ) =    5.3000D-01 * RXRAT(    83 )      ! BEN+HO=0.5300D+00*PHEN+...
      LOSS(  PHEN        ) =                 RXRAT(   123 )      ! PHEN+HO=0.1520D+00*ASOATJ+...
     &                     +                 RXRAT(   169 )      ! PHEN+NO3=0.1520D+00*ASOATJ+...

      PROD(  TOL         ) =   0.0D0
      LOSS(  TOL         ) =                 RXRAT(    84 )      ! TOL+HO=0.8200D+00*TOLP+...

      PROD(  TOLP        ) =    8.2000D-01 * RXRAT(    84 )      ! TOL+HO=0.8200D+00*TOLP+...
      LOSS(  TOLP        ) =                 RXRAT(   189 )      ! TOLP+NO=0.2000D-03*ONIT+...
     &                     +                 RXRAT(   231 )      ! TOLP+HO2=0.7195D+00*OP2+...
     &                     +                 RXRAT(   270 )      ! TOLP+MO2=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   307 )      ! TOLP+ACO3=0.7000D+00*MO2+...
     &                     +                 RXRAT(   346 )      ! TOLP+NO3=NO2+0.9146D+...

      PROD(  CSL         ) =    1.8000D-01 * RXRAT(    84 )      ! TOL+HO=0.1800D+00*CSL+...
     &                     +    1.7000D-01 * RXRAT(    85 )      ! XYM+HO=0.1700D+00*CSL+...
     &                     +    1.8000D-01 * RXRAT(    86 )      ! XYE+HO=0.1800D+00*CSL+...
     &                     +                 RXRAT(   255 )      ! CHO+HO2=CSL
      LOSS(  CSL         ) =                 RXRAT(   124 )      ! CSL+HO=0.2000D+00*ASOATJ+...
     &                     +                 RXRAT(   170 )      ! CSL+NO3=0.2000D+00*ASOATJ+...

      PROD(  XYM         ) =   0.0D0
      LOSS(  XYM         ) =                 RXRAT(    85 )      ! XYM+HO=0.8300D+00*XYMP+...

      PROD(  XYMP        ) =    8.3000D-01 * RXRAT(    85 )      ! XYM+HO=0.8300D+00*XYMP+...
      LOSS(  XYMP        ) =                 RXRAT(   190 )      ! XYMP+NO=0.1000D-03*ONIT+...
     &                     +                 RXRAT(   232 )      ! XYMP+HO2=0.4820D-01*OP2+...
     &                     +                 RXRAT(   271 )      ! XYMP+MO2=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   308 )      ! XYMP+ACO3=0.7000D+00*MO2+...
     &                     +                 RXRAT(   347 )      ! XYMP+NO3=NO2+0.9518D+...

      PROD(  XYE         ) =   0.0D0
      LOSS(  XYE         ) =                 RXRAT(    86 )      ! XYE+HO=0.8200D+00*XYEP+...

      PROD(  XYEP        ) =    8.2000D-01 * RXRAT(    86 )      ! XYE+HO=0.8200D+00*XYEP+...
      LOSS(  XYEP        ) =                 RXRAT(   191 )      ! XYEP+NO=0.2000D-03*ONIT+...
     &                     +                 RXRAT(   233 )      ! XYEP+HO2=0.8540D-01*OP2+...
     &                     +                 RXRAT(   272 )      ! XYEP+MO2=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   309 )      ! XYEP+ACO3=0.7000D+00*MO2+...
     &                     +                 RXRAT(   348 )      ! XYEP+NO3=NO2+0.9146D+...

      PROD(  ISO         ) =   0.0D0
      LOSS(  ISO         ) =                 RXRAT(    87 )      ! ISO+O3=0.5310D+00*MACR+...
     &                     +                 RXRAT(    88 )      ! ISO+NO3=INO2+0.3000D+...
     &                     +                 RXRAT(    89 )      ! ISO+HO=ISOP+0.2000D-01*MO2

      PROD(  INO2        ) =                 RXRAT(    88 )      ! ISO+NO3=INO2+0.3000D+...
      LOSS(  INO2        ) =                 RXRAT(    93 )      ! INO2+HO2=IPN+HO
     &                     +                 RXRAT(    94 )      ! INO2+NO=0.9000D+00*HCHO+...

      PROD(  ISON        ) =    3.0000D-01 * RXRAT(    88 )      ! ISO+NO3=0.3000D+00*ISON+...
     &                     +    2.0000D-01 * RXRAT(    94 )      ! INO2+NO=0.2000D+00*ISON+...
     &                     +    2.0000D-01 * RXRAT(    95 )      ! IPN+HO2=0.2000D+00*ISON+...
     &                     +    6.0000D-01 * RXRAT(    96 )      ! IHN=0.6000D+00*ISON+...
     &                     +    5.0000D-01 * RXRAT(    97 )      ! IHN+NO=0.5000D+00*ISON+...
      LOSS(  ISON        ) =                 RXRAT(   101 )      ! ISON+HO=CO+0.1200D+00*NO2
     &                     +                 RXRAT(   102 )      ! ISON+NO3=CO
     &                     +                 RXRAT(   420 )      ! ISON=AISONJ

      PROD(  ISOP        ) =                 RXRAT(    89 )      ! ISO+HO=ISOP+0.2000D-01*MO2
     &                     +                 RXRAT(    92 )      ! ISHP+HO=ISOP
      LOSS(  ISOP        ) =                 RXRAT(    90 )      ! ISOP+HO2=ISHP
     &                     +                 RXRAT(    91 )      ! ISOP+NO=IHN+0.3000D+...
     &                     +                 RXRAT(   234 )      ! ISOP+HO2=ISHP
     &                     +                 RXRAT(   273 )      ! ISOP+MO2=HO2+0.1310D+...
     &                     +                 RXRAT(   310 )      ! ISOP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   349 )      ! ISOP+NO3=HO2+NO2+0.7500D+...
     &                     +                 RXRAT(   377 )      ! ISOP+APIP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   380 )      ! ISOP+LIMP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   383 )      ! ISOP+APINP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   386 )      ! ISOP+LIMNP2=0.9600D+00*HOM+...

      PROD(  ISHP        ) =                 RXRAT(    90 )      ! ISOP+HO2=ISHP
     &                     +                 RXRAT(   234 )      ! ISOP+HO2=ISHP
      LOSS(  ISHP        ) =                 RXRAT(    92 )      ! ISHP+HO=ISOP
     &                     +                 RXRAT(    98 )      ! ISHP+HO=0.6000D+00*HCHO+...
     &                     +                 RXRAT(    99 )      ! ISHP=0.6000D+00*HCHO+...

      PROD(  IHN         ) =                 RXRAT(    91 )      ! ISOP+NO=IHN+0.3000D+...
      LOSS(  IHN         ) =                 RXRAT(    96 )      ! IHN=0.6000D+00*ISON+...
     &                     +                 RXRAT(    97 )      ! IHN+NO=0.6000D+00*HCHO+...

      PROD(  IPN         ) =                 RXRAT(    93 )      ! INO2+HO2=IPN+HO
      LOSS(  IPN         ) =                 RXRAT(    95 )      ! IPN+HO2=0.8000D+00*NO2+...

      PROD(  IEPOX       ) =    5.0000D-02 * RXRAT(    96 )      ! IHN=0.5000D-01*IEPOX+...
     &                     +    5.2000D-01 * RXRAT(    98 )      ! ISHP+HO=0.5200D+00*IEPOX+...
      LOSS(  IEPOX       ) =                 RXRAT(   387 )      ! IEPOX+HO=HO
     &                     +                 RXRAT(   425 )      ! IEPOX=IEPOXP

      PROD(  IPC         ) =    3.0000D-01 * RXRAT(    98 )      ! ISHP+HO=0.3000D+00*IPC+...
      LOSS(  IPC         ) =                 RXRAT(   100 )      ! IPC+NO=

      PROD(  API         ) =   0.0D0
      LOSS(  API         ) =                 RXRAT(   103 )      ! API+HO=0.9750D+00*APIP1+...
     &                     +                 RXRAT(   145 )      ! API+O3=0.9000D+00*HO+...
     &                     +                 RXRAT(   159 )      ! API+NO3=0.9750D+00*APINP1+...

      PROD(  APIP1       ) =    9.7500D-01 * RXRAT(   103 )      ! API+HO=0.9750D+00*APIP1+...
     &                     +    9.0000D-01 * RXRAT(   145 )      ! API+O3=0.9000D+00*APIP1+...
      LOSS(  APIP1       ) =                 RXRAT(   192 )      ! APIP1+NO=0.8200D+00*HO2+...
     &                     +                 RXRAT(   235 )      ! APIP1+HO2=OPB
     &                     +                 RXRAT(   274 )      ! APIP1+MO2=HO2+0.6800D+...
     &                     +                 RXRAT(   311 )      ! APIP1+ACO3=0.6300D+00*HO2+...
     &                     +                 RXRAT(   350 )      ! APIP1+NO3=HO2+NO2+ALD+KET
     &                     +                 RXRAT(   375 )      ! APIP1+APIP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   378 )      ! APIP1+LIMP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   381 )      ! APIP1+APINP2=0.9600D+...
     &                     +                 RXRAT(   384 )      ! APIP1+LIMNP2=0.9600D+...

      PROD(  APIP2       ) =    2.5000D-02 * RXRAT(   103 )      ! API+HO=0.2500D-01*APIP2+...
     &                     +    5.0000D-02 * RXRAT(   145 )      ! API+O3=0.5000D-01*APIP2+...
      LOSS(  APIP2       ) =                 RXRAT(   193 )      ! APIP2+NO=0.8200D+00*HO+...
     &                     +                 RXRAT(   236 )      ! APIP2+HO2=HOM
     &                     +                 RXRAT(   275 )      ! APIP2+MO2=HO2+0.7500D+...
     &                     +                 RXRAT(   312 )      ! APIP2+ACO3=0.5000D+00*HO+...
     &                     +                 RXRAT(   375 )      ! APIP2+APIP1=0.9600D+00*HOM+...
     &                     +                 RXRAT(   376 )      ! APIP2+LIMP1=0.9600D+00*HOM+...
     &                     +                 RXRAT(   377 )      ! APIP2+ISOP=0.9600D+00*HOM+...

      PROD(  LIM         ) =   0.0D0
      LOSS(  LIM         ) =                 RXRAT(   104 )      ! LIM+HO=0.9450D+00*LIMP1+...
     &                     +                 RXRAT(   146 )      ! LIM+O3=0.8400D+00*HO+...
     &                     +                 RXRAT(   160 )      ! LIM+NO3=0.9450D+00*LIMNP1+...

      PROD(  LIMP1       ) =    9.4500D-01 * RXRAT(   104 )      ! LIM+HO=0.9450D+00*LIMP1+...
     &                     +    8.4000D-01 * RXRAT(   146 )      ! LIM+O3=0.8400D+00*LIMP1+...
      LOSS(  LIMP1       ) =                 RXRAT(   196 )      ! LIMP1+NO=0.7700D+00*HO2+...
     &                     +                 RXRAT(   239 )      ! LIMP1+HO2=OPB
     &                     +                 RXRAT(   278 )      ! LIMP1+MO2=HO2+HCHO+0.4200D+...
     &                     +                 RXRAT(   315 )      ! LIMP1+ACO3=0.6300D+00*HO2+...
     &                     +                 RXRAT(   351 )      ! LIMP1+NO3=HO2+NO2+0.3850D+...
     &                     +                 RXRAT(   376 )      ! LIMP1+APIP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   379 )      ! LIMP1+LIMP2=0.9600D+00*HOM+...
     &                     +                 RXRAT(   382 )      ! LIMP1+APINP2=0.9600D+...
     &                     +                 RXRAT(   385 )      ! LIMP1+LIMNP2=0.9600D+...

      PROD(  LIMP2       ) =    5.5000D-02 * RXRAT(   104 )      ! LIM+HO=0.5500D-01*LIMP2+...
     &                     +    1.1000D-01 * RXRAT(   146 )      ! LIM+O3=0.1100D+00*LIMP2+...
      LOSS(  LIMP2       ) =                 RXRAT(   197 )      ! LIMP2+NO=0.7700D+00*HO+...
     &                     +                 RXRAT(   240 )      ! LIMP2+HO2=HOM
     &                     +                 RXRAT(   279 )      ! LIMP2+MO2=HO2+0.7500D+...
     &                     +                 RXRAT(   316 )      ! LIMP2+ACO3=0.5000D+00*HO+...
     &                     +                 RXRAT(   378 )      ! LIMP2+APIP1=0.9600D+00*HOM+...
     &                     +                 RXRAT(   379 )      ! LIMP2+LIMP1=0.9600D+00*HOM+...
     &                     +                 RXRAT(   380 )      ! LIMP2+ISOP=0.9600D+00*HOM+...

      PROD(  PINALP      ) =    2.3000D-01 * RXRAT(   105 )      ! PINAL+HO=0.2300D+00*PINALP+...
      LOSS(  PINALP      ) =                 RXRAT(   173 )      ! PINALP=HOM
     &                     +                 RXRAT(   200 )      ! PINALP+NO=0.9500D+00*HO2+...
     &                     +                 RXRAT(   243 )      ! PINALP+HO2=OPB

      PROD(  RCO3        ) =    7.7000D-01 * RXRAT(   105 )      ! PINAL+HO=0.7700D+00*RCO3+...
     &                     +    3.0000D-01 * RXRAT(   106 )      ! LIMAL+HO=0.3000D+00*RCO3+...
     &                     +                 RXRAT(   109 )      ! ALD+HO=RCO3
     &                     +    6.0000D-01 * RXRAT(   152 )      ! DCB1+O3=0.6000D+00*RCO3+...
     &                     +    6.0000D-01 * RXRAT(   153 )      ! DCB2+O3=0.6000D+00*RCO3+...
     &                     +                 RXRAT(   164 )      ! ALD+NO3=RCO3+HNO3
     &                     +                 RXRAT(   178 )      ! PPN=RCO3+NO2
      LOSS(  RCO3        ) =                 RXRAT(   177 )      ! RCO3+NO2=PPN
     &                     +                 RXRAT(   203 )      ! RCO3+NO=ETHP+NO2
     &                     +                 RXRAT(   246 )      ! RCO3+HO2=0.4400D+00*HO+...
     &                     +                 RXRAT(   283 )      ! RCO3+MO2=0.9000D+00*HO2+...
     &                     +                 RXRAT(   320 )      ! RCO3+ACO3=MO2+ETHP
     &                     +    2.0000D+00 * RXRAT(   337 )      ! RCO3+RCO3=0.2000D+01*ETHP
     &                     +                 RXRAT(   353 )      ! RCO3+NO3=ETHP+NO2
     &                     +                 RXRAT(   373 )      ! RCO3+XO2=ETHP

      PROD(  LIMALP      ) =    7.0000D-01 * RXRAT(   106 )      ! LIMAL+HO=0.7000D+00*LIMALP+...
      LOSS(  LIMALP      ) =                 RXRAT(   174 )      ! LIMALP=HOM
     &                     +                 RXRAT(   201 )      ! LIMALP+NO=0.9400D+00*HO2+...
     &                     +                 RXRAT(   244 )      ! LIMALP+HO2=OPB

      PROD(  ACTP        ) =                 RXRAT(   110 )      ! ACT+HO=ACTP
      LOSS(  ACTP        ) =                 RXRAT(   204 )      ! ACTP+NO=ACO3+NO2+HCHO
     &                     +                 RXRAT(   247 )      ! ACTP+HO2=0.1500D+00*HO+...
     &                     +                 RXRAT(   284 )      ! ACTP+MO2=0.5000D+00*HO2+...
     &                     +                 RXRAT(   321 )      ! ACTP+ACO3=0.5000D+00*MO2+...
     &                     +                 RXRAT(   354 )      ! ACTP+NO3=ACO3+NO2+HCHO

      PROD(  MEKP        ) =                 RXRAT(   111 )      ! MEK+HO=MEKP
     &                     +    2.5810D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.2581D+...
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    3.9000D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3900D-...
     &                     +    1.8400D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.1840D+...
     &                     +    6.4600D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.6460D-...
     &                     +    2.4400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2440D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
     &                     +    4.4500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4450D-...
     &                     +    8.1800D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.8180D-...
     &                     +    2.9700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2970D-...
     &                     +    6.2800D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6280D-...
     &                     +    6.9800D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6980D-...
     &                     +    7.8600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7860D-...
     &                     +    5.2600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.5260D-...
     &                     +    5.7100D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5710D-...
     &                     +    2.2660D-01 * RXRAT(   517 )      ! OP3+HO=0.2266D+00*MEKP+...
      LOSS(  MEKP        ) =                 RXRAT(   205 )      ! MEKP+NO=0.6700D+00*HO2+NO2+...
     &                     +                 RXRAT(   248 )      ! MEKP+HO2=OP2
     &                     +                 RXRAT(   285 )      ! MEKP+MO2=0.8340D+00*HO2+...
     &                     +                 RXRAT(   322 )      ! MEKP+ACO3=0.3300D+00*HO2+...
     &                     +                 RXRAT(   355 )      ! MEKP+NO3=0.6700D+00*HO2+...

      PROD(  KETP        ) =                 RXRAT(   112 )      ! KET+HO=KETP
     &                     +    7.9400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.7940D-...
     &                     +    1.0680D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1068D+...
      LOSS(  KETP        ) =                 RXRAT(   206 )      ! KETP+NO=0.7700D+00*HO2+...
     &                     +                 RXRAT(   249 )      ! KETP+HO2=OP2
     &                     +                 RXRAT(   286 )      ! KETP+MO2=HO2+0.7500D+...
     &                     +                 RXRAT(   323 )      ! KETP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   356 )      ! KETP+NO3=HO2+NO2+DCB1

      PROD(  MCP         ) =    4.3000D-01 * RXRAT(   114 )      ! MACR+HO=0.4300D+00*MCP+...
     &                     +    4.3000D-01 * RXRAT(   390 )      ! ACRO+HO=0.4300D+00*MCP+...
      LOSS(  MCP         ) =                 RXRAT(   208 )      ! MCP+NO=NO2+0.5000D+00*HO2+...
     &                     +                 RXRAT(   251 )      ! MCP+HO2=MAHP
     &                     +                 RXRAT(   288 )      ! MCP+MO2=NO2+HO2+0.1500D+...
     &                     +                 RXRAT(   325 )      ! MCP+ACO3=NO2+0.5000D+...
     &                     +                 RXRAT(   358 )      ! MCP+NO3=NO2+HO2+HCHO+HKET

      PROD(  MVKP        ) =                 RXRAT(   115 )      ! MVK+HO=MVKP
      LOSS(  MVKP        ) =                 RXRAT(   209 )      ! MVKP+NO=0.3000D+00*HO2+...
     &                     +                 RXRAT(   252 )      ! MVKP+HO2=OP2
     &                     +                 RXRAT(   289 )      ! MVKP+MO2=HO2+0.1160D+...
     &                     +                 RXRAT(   326 )      ! MVKP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   359 )      ! MVKP+NO3=0.3000D+00*HO2+...

      PROD(  UALP        ) =    6.8700D-01 * RXRAT(   116 )      ! UALD+HO=0.6870D+00*UALP+...
      LOSS(  UALP        ) =                 RXRAT(   210 )      ! UALP+NO=HO2+NO2+0.6100D+...
     &                     +                 RXRAT(   253 )      ! UALP+HO2=OP2
     &                     +                 RXRAT(   290 )      ! UALP+MO2=HO2+0.3050D+00*CO+...
     &                     +                 RXRAT(   327 )      ! UALP+ACO3=0.5000D+00*HO2+...
     &                     +                 RXRAT(   360 )      ! UALP+NO3=HO2+NO2+0.6100D+...

      PROD(  DCB3        ) =    2.4950D-01 * RXRAT(   188 )      ! BENP+NO=0.2495D+00*DCB3+...
     &                     +    2.5000D-01 * RXRAT(   269 )      ! BENP+MO2=0.2500D+00*DCB3+...
     &                     +    2.5000D-01 * RXRAT(   306 )      ! BENP+ACO3=0.2500D+00*DCB3+...
     &                     +    2.5000D-01 * RXRAT(   345 )      ! BENP+NO3=0.2500D+00*DCB3+...
     &                     +    2.0000D-01 * RXRAT(   407 )      ! FURAN+NO3=0.2000D+00*DCB3+...
      LOSS(  DCB3        ) =                 RXRAT(   121 )      ! DCB3+HO=0.5600D+00*HO2+...
     &                     +                 RXRAT(   154 )      ! DCB3+O3=0.5000D-01*HO+HO2+...

      PROD(  BALP        ) =                 RXRAT(   122 )      ! BALD+HO=BALP
      LOSS(  BALP        ) =                 RXRAT(   211 )      ! BALP+NO=BAL1+NO2
     &                     +                 RXRAT(   291 )      ! BALP+MO2=HO2+BAL1+HCHO
     &                     +                 RXRAT(   328 )      ! BALP+ACO3=MO2+BAL1
     &                     +                 RXRAT(   361 )      ! BALP+NO3=BAL1+NO2

      PROD(  ADDC        ) =    1.7000D-01 * RXRAT(   123 )      ! PHEN+HO=0.1700D+00*ADDC+...
     &                     +    1.6000D-01 * RXRAT(   124 )      ! CSL+HO=0.1600D+00*ADDC+...
     &                     +    8.5000D-01 * RXRAT(   169 )      ! PHEN+NO3=0.8500D+00*ADDC+...
     &                     +    8.0000D-02 * RXRAT(   170 )      ! CSL+NO3=0.8000D-01*ADDC+...
      LOSS(  ADDC        ) =                 RXRAT(   213 )      ! ADDC+NO=HO2+NO2+0.3200D+...
     &                     +                 RXRAT(   254 )      ! ADDC+HO2=OP2
     &                     +                 RXRAT(   293 )      ! ADDC+MO2=0.2000D+01*HO2+...
     &                     +                 RXRAT(   330 )      ! ADDC+ACO3=0.2000D+01*HO2+...
     &                     +                 RXRAT(   363 )      ! ADDC+NO3=HO2+NO2+0.3200D+...

      PROD(  MCT         ) =    6.1900D-01 * RXRAT(   123 )      ! PHEN+HO=0.6190D+00*MCT+...
     &                     +    5.8400D-01 * RXRAT(   124 )      ! CSL+HO=0.5840D+00*MCT+...
      LOSS(  MCT         ) =                 RXRAT(   125 )      ! MCT+HO=MCTO
     &                     +                 RXRAT(   171 )      ! MCT+NO3=MCTO+HNO3

      PROD(  MCTO        ) =                 RXRAT(   125 )      ! MCT+HO=MCTO
     &                     +                 RXRAT(   171 )      ! MCT+NO3=MCTO+HNO3
     &                     +                 RXRAT(   214 )      ! MCTP+NO=MCTO+NO2
     &                     +                 RXRAT(   294 )      ! MCTP+MO2=MCTO+HO2+HCHO
     &                     +                 RXRAT(   331 )      ! MCTP+ACO3=MCTO+MO2+HO2
     &                     +                 RXRAT(   364 )      ! MCTP+NO3=MCTO+NO2
      LOSS(  MCTO        ) =                 RXRAT(   155 )      ! MCTO+O3=MCTP
     &                     +                 RXRAT(   222 )      ! MCTO+NO2=ONIT

      PROD(  MOH         ) =    6.3000D-01 * RXRAT(   262 )      ! MO2+MO2=0.6300D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   263 )      ! ETHP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   264 )      ! HC3P+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   265 )      ! HC5P+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   266 )      ! ETEP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   267 )      ! OLTP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   268 )      ! OLIP+MO2=0.2500D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   269 )      ! BENP+MO2=0.3200D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   270 )      ! TOLP+MO2=0.3200D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   271 )      ! XYMP+MO2=0.3200D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   272 )      ! XYEP+MO2=0.3200D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   273 )      ! ISOP+MO2=0.2500D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   274 )      ! APIP1+MO2=0.3200D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   275 )      ! APIP2+MO2=0.2500D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   276 )      ! APINP1+MO2=0.3200D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   277 )      ! APINP2+MO2=0.2500D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   278 )      ! LIMP1+MO2=0.3200D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   279 )      ! LIMP2+MO2=0.2500D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   280 )      ! LIMNP1+MO2=0.3200D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   281 )      ! LIMNP2+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   284 )      ! ACTP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   285 )      ! MEKP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   286 )      ! KETP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   287 )      ! MACP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   288 )      ! MCP+MO2=0.2500D+00*MOH+HO2+...
     &                     +    2.5000D-01 * RXRAT(   289 )      ! MVKP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   290 )      ! UALP+MO2=0.2500D+00*MOH+...
     &                     +    2.5000D-01 * RXRAT(   297 )      ! OLND+MO2=0.2500D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   398 )      ! BDE13P+MO2=0.3200D+00*MOH+...
     &                     +    3.2000D-01 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.3200D+...
     &                     +    3.2000D-01 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.3200D+...
     &                     +    3.2000D-01 * RXRAT(   500 )      ! NAPHP+MO2=0.3200D+00*MOH+...
      LOSS(  MOH         ) =                 RXRAT(   126 )      ! MOH+HO=HO2+HCHO

      PROD(  EOH         ) =    2.5000D-01 * RXRAT(   263 )      ! ETHP+MO2=0.2500D+00*EOH+...
      LOSS(  EOH         ) =                 RXRAT(   127 )      ! EOH+HO=HO2+ACD

      PROD(  ROH         ) =    2.5000D-01 * RXRAT(   264 )      ! HC3P+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   265 )      ! HC5P+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   267 )      ! OLTP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   268 )      ! OLIP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   273 )      ! ISOP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   274 )      ! APIP1+MO2=0.2500D+00*ROH+...
     &                     +    2.7000D-01 * RXRAT(   278 )      ! LIMP1+MO2=0.2700D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   284 )      ! ACTP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   285 )      ! MEKP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   286 )      ! KETP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   287 )      ! MACP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   288 )      ! MCP+MO2=0.2500D+00*ROH+HO2+...
     &                     +    2.5000D-01 * RXRAT(   289 )      ! MVKP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   290 )      ! UALP+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   297 )      ! OLND+MO2=0.2500D+00*ROH+...
     &                     +    2.5000D-01 * RXRAT(   311 )      ! APIP1+ACO3=0.2500D+00*ROH+...
     &                     +    2.7000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.2700D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   375 )      ! APIP2+APIP1=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   376 )      ! APIP2+LIMP1=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   377 )      ! APIP2+ISOP=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   378 )      ! LIMP2+APIP1=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   379 )      ! LIMP2+LIMP1=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   380 )      ! LIMP2+ISOP=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   381 )      ! APINP2+APIP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   382 )      ! APINP2+LIMP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   383 )      ! APINP2+ISOP=0.4800D+00*ROH+...
     &                     +    4.8000D-01 * RXRAT(   384 )      ! LIMNP2+APIP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   385 )      ! LIMNP2+LIMP1=0.4800D+...
     &                     +    4.8000D-01 * RXRAT(   386 )      ! LIMNP2+ISOP=0.4800D+00*ROH+...
      LOSS(  ROH         ) =                 RXRAT(   128 )      ! ROH+HO=HO2+0.7190D+00*ALD+...

      PROD(  ETEG        ) =    2.5000D-01 * RXRAT(   266 )      ! ETEP+MO2=0.2500D+00*ETEG+...
      LOSS(  ETEG        ) =                 RXRAT(   129 )      ! ETEG+HO=HO2+ALD

      PROD(  MAHP        ) =                 RXRAT(   250 )      ! MACP+HO2=MAHP
     &                     +                 RXRAT(   251 )      ! MCP+HO2=MAHP
      LOSS(  MAHP        ) =                 RXRAT(   133 )      ! MAHP+HO=MACP

      PROD(  ORA2        ) =    6.0000D-02 * RXRAT(   143 )      ! OLT+O3=0.6000D-01*ORA2+...
     &                     +    1.7000D-02 * RXRAT(   144 )      ! OLI+O3=0.1700D-01*ORA2+...
     &                     +    7.0000D-02 * RXRAT(   150 )      ! MVK+O3=0.7000D-01*ORA2+...
     &                     +    2.5000D-01 * RXRAT(   154 )      ! DCB3+O3=0.2500D+00*ORA2+...
     &                     +    1.5000D-01 * RXRAT(   245 )      ! ACO3+HO2=0.1500D+00*ORA2+...
     &                     +    1.5000D-01 * RXRAT(   246 )      ! RCO3+HO2=0.1500D+00*ORA2+...
     &                     +    1.0000D-01 * RXRAT(   282 )      ! ACO3+MO2=0.1000D+00*ORA2+...
     &                     +    1.0000D-01 * RXRAT(   283 )      ! RCO3+MO2=0.1000D+00*ORA2+...
     &                     +    1.2500D-01 * RXRAT(   284 )      ! ACTP+MO2=0.1250D+00*ORA2+...
     &                     +    6.7000D-02 * RXRAT(   287 )      ! MACP+MO2=0.6700D-01*ORA2+...
     &                     +    2.9200D-01 * RXRAT(   289 )      ! MVKP+MO2=0.2920D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   300 )      ! ETHP+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   301 )      ! HC3P+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   303 )      ! ETEP+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   304 )      ! OLTP+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   305 )      ! OLIP+ACO3=0.5000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   306 )      ! BENP+ACO3=0.3000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.3000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.3000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.3000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   310 )      ! ISOP+ACO3=0.5000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   311 )      ! APIP1+ACO3=0.3000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   312 )      ! APIP2+ACO3=0.5000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   313 )      ! APINP1+ACO3=0.3000D+...
     &                     +    5.0000D-01 * RXRAT(   314 )      ! APINP2+ACO3=0.5000D+...
     &                     +    3.0000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.3000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   316 )      ! LIMP2+ACO3=0.5000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   317 )      ! LIMNP1+ACO3=0.3000D+...
     &                     +    5.0000D-01 * RXRAT(   318 )      ! LIMNP2+ACO3=0.5000D+...
     &                     +    7.5000D-01 * RXRAT(   321 )      ! ACTP+ACO3=0.7500D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   322 )      ! MEKP+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   323 )      ! KETP+ACO3=0.5000D+00*ORA2+...
     &                     +    6.3500D-01 * RXRAT(   324 )      ! MACP+ACO3=0.6350D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   325 )      ! MCP+ACO3=0.5000D+00*ORA2+...
     &                     +    1.0830D+00 * RXRAT(   326 )      ! MVKP+ACO3=0.1083D+01*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.5000D+00*ORA2+...
     &                     +    5.0000D-01 * RXRAT(   334 )      ! OLND+ACO3=0.5000D+00*ORA2+...
     &                     +    3.0000D-01 * RXRAT(   399 )      ! BDE13P+ACO3=0.3000D+...
     &                     +    3.0000D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.3000D+...
     &                     +    3.0000D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.3000D+...
     &                     +    3.0000D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.3000D+00*ORA2+...
      LOSS(  ORA2        ) =                 RXRAT(   135 )      ! ORA2+HO=0.6400D+00*MO2+...

      PROD(  ORAP        ) =    3.6000D-01 * RXRAT(   135 )      ! ORA2+HO=0.3600D+00*ORAP+...
      LOSS(  ORAP        ) =                 RXRAT(   215 )      ! ORAP+NO=NO2+GLY+HO2
     &                     +                 RXRAT(   257 )      ! ORAP+HO2=OP2
     &                     +                 RXRAT(   295 )      ! ORAP+MO2=HCHO+HO2+GLY
     &                     +                 RXRAT(   332 )      ! ORAP+ACO3=MO2+GLY
     &                     +                 RXRAT(   365 )      ! ORAP+NO3=NO2+GLY+HO2

      PROD(  PPN         ) =                 RXRAT(   177 )      ! RCO3+NO2=PPN
      LOSS(  PPN         ) =                 RXRAT(   138 )      ! PPN+HO=XO2+NO3+HCHO
     &                     +                 RXRAT(   178 )      ! PPN=RCO3+NO2

      PROD(  MPAN        ) =                 RXRAT(   179 )      ! MACP+NO2=MPAN
      LOSS(  MPAN        ) =                 RXRAT(   139 )      ! MPAN+HO=NO2+HKET
     &                     +                 RXRAT(   172 )      ! MPAN+NO3=MACP+NO2
     &                     +                 RXRAT(   180 )      ! MPAN=MACP+NO2

      PROD(  TRPN        ) =    1.8000D-01 * RXRAT(   192 )      ! APIP1+NO=0.1800D+00*TRPN+...
     &                     +    2.3000D-01 * RXRAT(   196 )      ! LIMP1+NO=0.2300D+00*TRPN+...
     &                     +    5.0000D-02 * RXRAT(   200 )      ! PINALP+NO=0.5000D-01*TRPN+...
     &                     +    6.0000D-02 * RXRAT(   201 )      ! LIMALP+NO=0.6000D-01*TRPN+...
     &                     +                 RXRAT(   237 )      ! APINP1+HO2=TRPN
     &                     +                 RXRAT(   241 )      ! LIMNP1+HO2=TRPN
     &                     +    1.4000D-01 * RXRAT(   276 )      ! APINP1+MO2=0.1400D+00*TRPN+...
     &                     +    3.0000D-01 * RXRAT(   280 )      ! LIMNP1+MO2=0.3000D+00*TRPN+...
     &                     +    1.4000D-01 * RXRAT(   313 )      ! APINP1+ACO3=0.1400D+...
     &                     +    3.0000D-01 * RXRAT(   317 )      ! LIMNP1+ACO3=0.3000D+...
      LOSS(  TRPN        ) =                 RXRAT(   141 )      ! TRPN+HO=HOM
     &                     +                 RXRAT(   148 )      ! TRPN+O3=HOM
     &                     +                 RXRAT(   161 )      ! TRPN+NO3=HOM
     &                     +                 RXRAT(   421 )      ! TRPN=ATRPNJ

      PROD(  HOM         ) =                 RXRAT(   141 )      ! TRPN+HO=HOM
     &                     +                 RXRAT(   148 )      ! TRPN+O3=HOM
     &                     +                 RXRAT(   161 )      ! TRPN+NO3=HOM
     &                     +                 RXRAT(   173 )      ! PINALP=HOM
     &                     +                 RXRAT(   174 )      ! LIMALP=HOM
     &                     +                 RXRAT(   193 )      ! APIP2+NO=HOM+0.8200D+...
     &                     +                 RXRAT(   195 )      ! APINP2+NO=HOM+0.8200D+...
     &                     +                 RXRAT(   197 )      ! LIMP2+NO=HOM+0.7700D+...
     &                     +                 RXRAT(   199 )      ! LIMNP2+NO=HOM+0.7700D+...
     &                     +                 RXRAT(   236 )      ! APIP2+HO2=HOM
     &                     +                 RXRAT(   238 )      ! APINP2+HO2=HOM
     &                     +                 RXRAT(   240 )      ! LIMP2+HO2=HOM
     &                     +                 RXRAT(   242 )      ! LIMNP2+HO2=HOM
     &                     +                 RXRAT(   275 )      ! APIP2+MO2=HOM+0.7500D+...
     &                     +                 RXRAT(   277 )      ! APINP2+MO2=HOM+0.7500D+...
     &                     +                 RXRAT(   279 )      ! LIMP2+MO2=HOM+0.7500D+...
     &                     +                 RXRAT(   281 )      ! LIMNP2+MO2=HOM+0.7500D+...
     &                     +                 RXRAT(   312 )      ! APIP2+ACO3=HOM+0.5000D+...
     &                     +                 RXRAT(   314 )      ! APINP2+ACO3=HOM+0.5000D+...
     &                     +                 RXRAT(   316 )      ! LIMP2+ACO3=HOM+0.5000D+...
     &                     +                 RXRAT(   318 )      ! LIMNP2+ACO3=HOM+0.5000D+...
     &                     +    9.6000D-01 * RXRAT(   375 )      ! APIP2+APIP1=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   376 )      ! APIP2+LIMP1=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   377 )      ! APIP2+ISOP=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   378 )      ! LIMP2+APIP1=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   379 )      ! LIMP2+LIMP1=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   380 )      ! LIMP2+ISOP=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   381 )      ! APINP2+APIP1=0.9600D+...
     &                     +    9.6000D-01 * RXRAT(   382 )      ! APINP2+LIMP1=0.9600D+...
     &                     +    9.6000D-01 * RXRAT(   383 )      ! APINP2+ISOP=0.9600D+00*HOM+...
     &                     +    9.6000D-01 * RXRAT(   384 )      ! LIMNP2+APIP1=0.9600D+...
     &                     +    9.6000D-01 * RXRAT(   385 )      ! LIMNP2+LIMP1=0.9600D+...
     &                     +    9.6000D-01 * RXRAT(   386 )      ! LIMNP2+ISOP=0.9600D+00*HOM+...
      LOSS(  HOM         ) =   0.0D0

      PROD(  MCTP        ) =                 RXRAT(   155 )      ! MCTO+O3=MCTP
      LOSS(  MCTP        ) =                 RXRAT(   214 )      ! MCTP+NO=MCTO+NO2
     &                     +                 RXRAT(   256 )      ! MCTP+HO2=OP2
     &                     +                 RXRAT(   294 )      ! MCTP+MO2=HO2+MCTO+HCHO
     &                     +                 RXRAT(   331 )      ! MCTP+ACO3=HO2+MO2+MCTO
     &                     +                 RXRAT(   364 )      ! MCTP+NO3=NO2+MCTO

      PROD(  OLNN        ) =    8.0000D-01 * RXRAT(   156 )      ! ETE+NO3=0.8000D+00*OLNN+...
     &                     +    4.3000D-01 * RXRAT(   157 )      ! OLT+NO3=0.4300D+00*OLNN+...
     &                     +    1.1000D-01 * RXRAT(   158 )      ! OLI+NO3=0.1100D+00*OLNN+...
     &                     +    9.0000D-01 * RXRAT(   401 )      ! BDE13+NO3=0.9000D+00*OLNN+...
      LOSS(  OLNN        ) =                 RXRAT(   216 )      ! OLNN+NO=NO2+HO2+ONIT
     &                     +                 RXRAT(   258 )      ! OLNN+HO2=ONIT
     &                     +                 RXRAT(   296 )      ! OLNN+MO2=0.2000D+01*HO2+...
     &                     +                 RXRAT(   333 )      ! OLNN+ACO3=HO2+MO2+ONIT
     &                     +                 RXRAT(   366 )      ! OLNN+NO3=HO2+NO2+ONIT
     &                     +    2.0000D+00 * RXRAT(   369 )      ! OLNN+OLNN=HO2+0.2000D+01*ONIT
     &                     +                 RXRAT(   370 )      ! OLNN+OLND=0.5000D+00*HO2+...

      PROD(  OLND        ) =    2.0000D-01 * RXRAT(   156 )      ! ETE+NO3=0.2000D+00*OLND+...
     &                     +    5.7000D-01 * RXRAT(   157 )      ! OLT+NO3=0.5700D+00*OLND+...
     &                     +    8.9000D-01 * RXRAT(   158 )      ! OLI+NO3=0.8900D+00*OLND+...
     &                     +    1.0000D-01 * RXRAT(   401 )      ! BDE13+NO3=0.1000D+00*OLND+...
      LOSS(  OLND        ) =                 RXRAT(   217 )      ! OLND+NO=0.2000D+01*NO2+...
     &                     +                 RXRAT(   259 )      ! OLND+HO2=ONIT
     &                     +                 RXRAT(   297 )      ! OLND+MO2=0.5000D+00*HO2+...
     &                     +                 RXRAT(   334 )      ! OLND+ACO3=0.5000D+00*MO2+...
     &                     +                 RXRAT(   367 )      ! OLND+NO3=0.2000D+01*NO2+...
     &                     +                 RXRAT(   370 )      ! OLND+OLNN=0.5000D+00*HO2+...
     &                     +    2.0000D+00 * RXRAT(   371 )      ! OLND+OLND=NO2+0.5040D+...

      PROD(  APINP1      ) =    9.7500D-01 * RXRAT(   159 )      ! API+NO3=0.9750D+00*APINP1+...
      LOSS(  APINP1      ) =                 RXRAT(   194 )      ! APINP1+NO=0.2000D+01*NO2+PINAL
     &                     +                 RXRAT(   237 )      ! APINP1+HO2=TRPN
     &                     +                 RXRAT(   276 )      ! APINP1+MO2=0.3700D+00*HO2+...
     &                     +                 RXRAT(   313 )      ! APINP1+ACO3=0.5000D+00*NO2+...

      PROD(  APINP2      ) =    2.5000D-02 * RXRAT(   159 )      ! API+NO3=0.2500D-01*APINP2+...
      LOSS(  APINP2      ) =                 RXRAT(   195 )      ! APINP2+NO=0.8200D+00*NO2+...
     &                     +                 RXRAT(   238 )      ! APINP2+HO2=HOM
     &                     +                 RXRAT(   277 )      ! APINP2+MO2=0.7500D+00*HO2+...
     &                     +                 RXRAT(   314 )      ! APINP2+ACO3=0.5000D+00*NO2+...
     &                     +                 RXRAT(   381 )      ! APINP2+APIP1=0.9600D+...
     &                     +                 RXRAT(   382 )      ! APINP2+LIMP1=0.9600D+...
     &                     +                 RXRAT(   383 )      ! APINP2+ISOP=0.9600D+00*HOM+...

      PROD(  LIMNP1      ) =    9.4500D-01 * RXRAT(   160 )      ! LIM+NO3=0.9450D+00*LIMNP1+...
      LOSS(  LIMNP1      ) =                 RXRAT(   198 )      ! LIMNP1+NO=0.2000D+01*NO2+LIMAL
     &                     +                 RXRAT(   241 )      ! LIMNP1+HO2=TRPN
     &                     +                 RXRAT(   280 )      ! LIMNP1+MO2=0.3700D+00*HO2+...
     &                     +                 RXRAT(   317 )      ! LIMNP1+ACO3=0.7000D+00*NO2+...

      PROD(  LIMNP2      ) =    5.5000D-02 * RXRAT(   160 )      ! LIM+NO3=0.5500D-01*LIMNP2+...
      LOSS(  LIMNP2      ) =                 RXRAT(   199 )      ! LIMNP2+NO=0.7700D+00*NO2+...
     &                     +                 RXRAT(   242 )      ! LIMNP2+HO2=HOM
     &                     +                 RXRAT(   281 )      ! LIMNP2+MO2=0.7500D+00*HO2+...
     &                     +                 RXRAT(   318 )      ! LIMNP2+ACO3=0.5000D+00*MO2+...
     &                     +                 RXRAT(   384 )      ! LIMNP2+APIP1=0.9600D+...
     &                     +                 RXRAT(   385 )      ! LIMNP2+LIMP1=0.9600D+...
     &                     +                 RXRAT(   386 )      ! LIMNP2+ISOP=0.9600D+00*HOM+...

      PROD(  ADCN        ) =    4.2400D-01 * RXRAT(   169 )      ! PHEN+NO3=0.4240D+00*ADCN+...
     &                     +    4.0000D-01 * RXRAT(   170 )      ! CSL+NO3=0.4000D+00*ADCN+...
      LOSS(  ADCN        ) =                 RXRAT(   218 )      ! ADCN+NO=0.2000D+01*NO2+GLY+OP2
     &                     +                 RXRAT(   260 )      ! ADCN+HO2=OP2
     &                     +                 RXRAT(   298 )      ! ADCN+MO2=HO2+0.7000D+...
     &                     +                 RXRAT(   335 )      ! ADCN+ACO3=HO2+MO2+0.7000D+...
     &                     +                 RXRAT(   368 )      ! ADCN+NO3=0.2000D+01*NO2+...

      PROD(  VROCP4OXY2  ) =    1.2000D-03 * RXRAT(   188 )      ! BENP+NO=0.1200D-...
     &                     +    1.3000D-03 * RXRAT(   189 )      ! TOLP+NO=0.1300D-...
     &                     +    2.8000D-01 * RXRAT(   435 )      ! VROCP6ALKP+NO=0.2800D+...
     &                     +                 RXRAT(   462 )      ! HC10P2=VROCP4OXY2+HO2
     &                     +    1.6000D-01 * RXRAT(   484 )      ! VROCP6ARO+HO=0.1600D+...
     &                     +    5.9500D-02 * RXRAT(   485 )      ! VROCP6AROP+HO2=0.5950D-...
     &                     +    1.0000D-04 * RXRAT(   486 )      ! VROCP6AROP+NO=0.1000D-...
     &                     +    5.9400D-02 * RXRAT(   492 )      ! VROCP5AROP+NO=0.5940D-...
     &                     +    5.9500D-02 * RXRAT(   493 )      ! VROCP5AROP+NO3=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   498 )      ! NAPHP+NO=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   499 )      ! NAPHP+NO3=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   500 )      ! NAPHP+MO2=0.5950D-...
     &                     +    5.9500D-02 * RXRAT(   501 )      ! NAPHP+ACO3=0.5950D-...
     &                     +    1.8300D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.1830D-...
     &                     +    4.5500D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.4550D-...
     &                     +    5.1200D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.5120D-...
     &                     +    5.0900D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.5090D-...
     &                     +    2.4600D-02 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.2460D-...
     &                     +    5.1900D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.5190D-...
     &                     +    6.4900D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.6490D-...
     &                     +    6.6800D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.6680D-...
     &                     +    7.3300D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.7330D-...
     &                     +    4.2600D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.4260D-...
     &                     +    7.2400D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.7240D-...
     &                     +    1.4630D-01 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.1463D+...
      LOSS(  VROCP4OXY2  ) =    9.9760D-01 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.2400D-...

      PROD(  VROCN1OXY6  ) =    8.0000D-04 * RXRAT(   188 )      ! BENP+NO=0.8000D-...
     &                     +    6.0000D-04 * RXRAT(   189 )      ! TOLP+NO=0.6000D-...
     &                     +    3.9790D-01 * RXRAT(   230 )      ! BENP+HO2=0.3979D+...
     &                     +    2.8050D-01 * RXRAT(   231 )      ! TOLP+HO2=0.2805D+...
     &                     +    1.2100D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.1210D-...
     &                     +    1.5300D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.1530D-...
     &                     +    3.1900D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.3190D-...
     &                     +    1.2200D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.1220D-...
     &                     +    3.2000D-02 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.3200D-...
     &                     +    3.2500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.3250D-...
     &                     +    7.4000D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.7400D-...
     &                     +    8.0000D-03 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.8000D-...
     &                     +    5.1300D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.5130D-...
     &                     +    5.2600D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.5260D-...
     &                     +    4.8000D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.4800D-...
     &                     +    9.0000D-03 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.9000D-...
     &                     +    6.1000D-03 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.6100D-...
     &                     +    3.9000D-02 * RXRAT(   517 )      ! OP3+HO=0.3900D-...
      LOSS(  VROCN1OXY6  ) =                 RXRAT(   505 )      ! VROCN1OXY6+HO=HO+0.2036D+...

      PROD(  FURANONE    ) =    4.9900D-01 * RXRAT(   188 )      ! BENP+NO=0.4990D+...
     &                     +    3.6510D-01 * RXRAT(   189 )      ! TOLP+NO=0.3651D+...
     &                     +    3.5150D-01 * RXRAT(   190 )      ! XYMP+NO=0.3515D+...
     &                     +    4.5640D-01 * RXRAT(   191 )      ! XYEP+NO=0.4564D+...
     &                     +    5.0000D-01 * RXRAT(   269 )      ! BENP+MO2=0.5000D+...
     &                     +    3.6590D-01 * RXRAT(   270 )      ! TOLP+MO2=0.3659D+...
     &                     +    3.5220D-01 * RXRAT(   271 )      ! XYMP+MO2=0.3522D+...
     &                     +    4.5730D-01 * RXRAT(   272 )      ! XYEP+MO2=0.4573D+...
     &                     +    5.0000D-01 * RXRAT(   306 )      ! BENP+ACO3=0.5000D+...
     &                     +    3.6590D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.3659D+...
     &                     +    3.5220D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.3522D+...
     &                     +    4.5730D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.4573D+...
     &                     +    5.0000D-01 * RXRAT(   345 )      ! BENP+NO3=0.5000D+...
     &                     +    3.6590D-01 * RXRAT(   346 )      ! TOLP+NO3=0.3659D+...
     &                     +    3.5220D-01 * RXRAT(   347 )      ! XYMP+NO3=0.3522D+...
     &                     +    4.5730D-01 * RXRAT(   348 )      ! XYEP+NO3=0.4573D+...
     &                     +    9.2000D-01 * RXRAT(   403 )      ! FURANO2+NO=0.9200D+...
     &                     +    4.0000D-01 * RXRAT(   404 )      ! FURANO2+HO2=0.4000D+...
     &                     +    4.6930D-01 * RXRAT(   486 )      ! VROCP6AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   487 )      ! VROCP6AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   492 )      ! VROCP5AROP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   493 )      ! VROCP5AROP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.4702D+...
     &                     +    4.6930D-01 * RXRAT(   498 )      ! NAPHP+NO=0.4693D+...
     &                     +    4.7020D-01 * RXRAT(   499 )      ! NAPHP+NO3=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   500 )      ! NAPHP+MO2=0.4702D+...
     &                     +    4.7020D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.4702D+...
      LOSS(  FURANONE    ) =                 RXRAT(   405 )      ! FURANONE+HO=0.6500D+00*KET+...

      PROD(  VROCP3OXY2  ) =    1.3000D-03 * RXRAT(   190 )      ! XYMP+NO=0.1300D-...
     &                     +    1.3000D-03 * RXRAT(   191 )      ! XYEP+NO=0.1300D-...
     &                     +                 RXRAT(   411 )      ! SESQNRO2+NO=VROCP3OXY2+...
     &                     +                 RXRAT(   412 )      ! SESQNRO2+NO3=VROCP3OXY2+...
     &                     +    9.8200D-01 * RXRAT(   413 )      ! SESQ+O3=0.9820D+...
     &                     +                 RXRAT(   416 )      ! SESQRO2+NO3=VROCP3OXY2
     &                     +    7.5300D-01 * RXRAT(   417 )      ! SESQRO2+NO=0.7530D+...
     &                     +    2.8000D-01 * RXRAT(   436 )      ! VROCP5ALKP+NO=0.2800D+...
     &                     +                 RXRAT(   449 )      ! VROCP6ALKP+HO2=VROCP3OXY2
     &                     +                 RXRAT(   456 )      ! VROCP6ALKP2=VROCP3OXY2+HO2
     &                     +    8.6000D-01 * RXRAT(   463 )      ! VROCP6ALKP2+NO=0.8600D+...
     &                     +                 RXRAT(   470 )      ! VROCP6ALKP2+NO3=VROCP3OXY2+...
     &                     +    1.6000D-01 * RXRAT(   490 )      ! VROCP5ARO+HO=0.1600D+...
     &                     +    5.9500D-02 * RXRAT(   491 )      ! VROCP5AROP+HO2=0.5950D-...
     &                     +    1.0000D-04 * RXRAT(   492 )      ! VROCP5AROP+NO=0.1000D-...
     &                     +    1.6000D-01 * RXRAT(   496 )      ! NAPH+HO=0.1600D+...
     &                     +    5.9500D-02 * RXRAT(   497 )      ! NAPHP+HO2=0.5950D-...
     &                     +    3.9800D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.3980D-...
     &                     +    4.5200D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.4520D-...
     &                     +    4.6200D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.4620D-...
     &                     +    5.1800D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.5180D-...
     &                     +    5.3500D-02 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.5350D-...
     &                     +    5.1600D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.5160D-...
     &                     +    5.8500D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.5850D-...
     &                     +    7.0900D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.7090D-...
     &                     +    2.9300D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.2930D-...
     &                     +    4.5600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.4560D-...
     &                     +    1.5660D-01 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1566D+...
     &                     +    1.3840D-01 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.1384D+...
      LOSS(  VROCP3OXY2  ) =    9.9140D-01 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.8600D-...

      PROD(  VROCP0OXY4  ) =    6.0000D-04 * RXRAT(   190 )      ! XYMP+NO=0.6000D-...
     &                     +    6.0000D-04 * RXRAT(   191 )      ! XYEP+NO=0.6000D-...
     &                     +    2.7710D-01 * RXRAT(   232 )      ! XYMP+HO2=0.2771D+...
     &                     +    2.8050D-01 * RXRAT(   233 )      ! XYEP+HO2=0.2805D+...
     &                     +                 RXRAT(   410 )      ! SESQNRO2+HO2=VROCP0OXY4
     &                     +    6.2400D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.6240D-...
     &                     +    1.8900D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.1890D-...
     &                     +    1.9400D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.1940D-...
     &                     +    2.9100D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2910D-...
     &                     +    3.0700D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.3070D-...
     &                     +    6.3100D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.6310D-...
     &                     +    2.9200D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2920D-...
     &                     +    1.3370D-01 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.1337D+...
     &                     +    1.5500D-01 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.1550D+...
     &                     +    8.8100D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.8810D-...
     &                     +    7.0200D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.7020D-...
     &                     +    2.2400D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.2240D-...
     &                     +    1.1400D-02 * RXRAT(   517 )      ! OP3+HO=0.1140D-...
      LOSS(  VROCP0OXY4  ) =    9.9890D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1100D-...

      PROD(  BAL1        ) =                 RXRAT(   211 )      ! BALP+NO=BAL1+NO2
     &                     +                 RXRAT(   291 )      ! BALP+MO2=BAL1+HO2+HCHO
     &                     +                 RXRAT(   328 )      ! BALP+ACO3=BAL1+MO2
     &                     +                 RXRAT(   361 )      ! BALP+NO3=BAL1+NO2
      LOSS(  BAL1        ) =                 RXRAT(   212 )      ! BAL1+NO=BAL2+NO2
     &                     +                 RXRAT(   292 )      ! BAL1+MO2=HO2+BAL2+HCHO
     &                     +                 RXRAT(   329 )      ! BAL1+ACO3=MO2+BAL2
     &                     +                 RXRAT(   362 )      ! BAL1+NO3=BAL2+NO2

      PROD(  BAL2        ) =                 RXRAT(   212 )      ! BAL1+NO=BAL2+NO2
     &                     +                 RXRAT(   292 )      ! BAL1+MO2=BAL2+HO2+HCHO
     &                     +                 RXRAT(   329 )      ! BAL1+ACO3=BAL2+MO2
     &                     +                 RXRAT(   362 )      ! BAL1+NO3=BAL2+NO2
      LOSS(  BAL2        ) =                 RXRAT(   220 )      ! BAL2+NO2=ONIT

      PROD(  OP3         ) =    6.7470D-01 * RXRAT(   232 )      ! XYMP+HO2=0.6747D+00*OP3+...
     &                     +    6.3410D-01 * RXRAT(   233 )      ! XYEP+HO2=0.6341D+00*OP3+...
     &                     +    3.0800D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.3080D-...
     &                     +    2.6000D-03 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.2600D-...
     &                     +    1.6300D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.1630D-...
     &                     +    1.0500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.1050D-...
     &                     +    1.5400D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.1540D-...
     &                     +    6.6000D-03 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.6600D-...
     &                     +    4.1000D-03 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.4100D-...
     &                     +    5.0600D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.5060D-...
     &                     +    6.0700D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.6070D-...
     &                     +    2.1600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.2160D-...
     &                     +    3.1600D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.3160D-...
      LOSS(  OP3         ) =                 RXRAT(   517 )      ! OP3+HO=HO+0.1188D+...

      PROD(  ELHOM       ) =    4.0000D-02 * RXRAT(   375 )      ! APIP2+APIP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   376 )      ! APIP2+LIMP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   377 )      ! APIP2+ISOP=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   378 )      ! LIMP2+APIP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   379 )      ! LIMP2+LIMP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   380 )      ! LIMP2+ISOP=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   381 )      ! APINP2+APIP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   382 )      ! APINP2+LIMP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   383 )      ! APINP2+ISOP=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   384 )      ! LIMNP2+APIP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   385 )      ! LIMNP2+LIMP1=0.4000D-...
     &                     +    4.0000D-02 * RXRAT(   386 )      ! LIMNP2+ISOP=0.4000D-...
      LOSS(  ELHOM       ) =   0.0D0

      PROD(  VROCIOXY    ) =   0.0D0
      LOSS(  VROCIOXY    ) =                 RXRAT(   388 )      ! VROCIOXY+HO=0.8520D+...

      PROD(  SLOWROC     ) =   0.0D0
      LOSS(  SLOWROC     ) =                 RXRAT(   389 )      ! SLOWROC+HO=ETHP+0.1010D-...

      PROD(  ACRO        ) =    8.9500D-01 * RXRAT(   395 )      ! BDE13P+NO=0.8950D+00*ACRO+...
     &                     +    9.2500D-01 * RXRAT(   396 )      ! BDE13P+NO3=0.9250D+00*ACRO+...
     &                     +    4.6300D-01 * RXRAT(   398 )      ! BDE13P+MO2=0.4630D+00*ACRO+...
     &                     +    7.4000D-01 * RXRAT(   399 )      ! BDE13P+ACO3=0.7400D+...
     &                     +    6.2000D-01 * RXRAT(   400 )      ! BDE13+O3=0.6200D+00*ACRO+...
      LOSS(  ACRO        ) =                 RXRAT(   390 )      ! ACRO+HO=0.5700D+00*MACP+...
     &                     +                 RXRAT(   391 )      ! ACRO+O3=0.8400D+00*CO+...
     &                     +                 RXRAT(   392 )      ! ACRO+NO3=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   393 )      ! ACRO=CO+0.4770D+00*HO2+...

      PROD(  BDE13       ) =   0.0D0
      LOSS(  BDE13       ) =                 RXRAT(   394 )      ! BDE13+HO=0.6670D+00*BDE13P+...
     &                     +                 RXRAT(   400 )      ! BDE13+O3=0.6200D+00*ACRO+...
     &                     +                 RXRAT(   401 )      ! BDE13+NO3=0.9000D+00*OLNN+...

      PROD(  BDE13P      ) =    6.6700D-01 * RXRAT(   394 )      ! BDE13+HO=0.6670D+00*BDE13P+...
      LOSS(  BDE13P      ) =                 RXRAT(   395 )      ! BDE13P+NO=0.9680D+00*HO2+...
     &                     +                 RXRAT(   396 )      ! BDE13P+NO3=HO2+NO2+0.9250D+...
     &                     +                 RXRAT(   397 )      ! BDE13P+HO2=OP2
     &                     +                 RXRAT(   398 )      ! BDE13P+MO2=0.3200D+00*MOH+...
     &                     +                 RXRAT(   399 )      ! BDE13P+ACO3=0.7000D+00*MO2+...

      PROD(  FURAN       ) =    7.2000D-02 * RXRAT(   395 )      ! BDE13P+NO=0.7200D-01*FURAN+...
     &                     +    7.5000D-02 * RXRAT(   396 )      ! BDE13P+NO3=0.7500D-...
     &                     +    3.7000D-02 * RXRAT(   398 )      ! BDE13P+MO2=0.3700D-...
     &                     +    6.0000D-02 * RXRAT(   399 )      ! BDE13P+ACO3=0.6000D-...
      LOSS(  FURAN       ) =                 RXRAT(   402 )      ! FURAN+HO=0.4900D+00*DCB1+...
     &                     +                 RXRAT(   406 )      ! FURAN+O3=0.2000D-01*HO+ALD
     &                     +                 RXRAT(   407 )      ! FURAN+NO3=NO2+0.8000D+...

      PROD(  FURANO2     ) =    5.1000D-01 * RXRAT(   402 )      ! FURAN+HO=0.5100D+...
      LOSS(  FURANO2     ) =                 RXRAT(   403 )      ! FURANO2+NO=0.8000D-01*ONIT+...
     &                     +                 RXRAT(   404 )      ! FURANO2+HO2=0.6000D+00*OP2+...

      PROD(  PROG        ) =   0.0D0
      LOSS(  PROG        ) =                 RXRAT(   408 )      ! PROG+HO=0.6130D+00*HKET+...

      PROD(  SESQ        ) =   0.0D0
      LOSS(  SESQ        ) =                 RXRAT(   409 )      ! SESQ+NO3=SESQNRO2
     &                     +                 RXRAT(   413 )      ! SESQ+O3=0.9820D+...
     &                     +                 RXRAT(   414 )      ! SESQ+HO=SESQRO2

      PROD(  SESQNRO2    ) =                 RXRAT(   409 )      ! SESQ+NO3=SESQNRO2
      LOSS(  SESQNRO2    ) =                 RXRAT(   410 )      ! SESQNRO2+HO2=VROCP0OXY4
     &                     +                 RXRAT(   411 )      ! SESQNRO2+NO=VROCP3OXY2+...
     &                     +                 RXRAT(   412 )      ! SESQNRO2+NO3=VROCP3OXY2+...

      PROD(  VROCN2OXY2  ) =    1.8000D-02 * RXRAT(   413 )      ! SESQ+O3=0.1800D-...
     &                     +                 RXRAT(   454 )      ! VROCP1ALKP+HO2=VROCN2OXY2
     &                     +                 RXRAT(   461 )      ! VROCP1ALKP2=VROCN2OXY2+HO2
     &                     +    1.4000D-01 * RXRAT(   467 )      ! VROCP2ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   468 )      ! VROCP1ALKP2+NO=VROCN2OXY2+...
     &                     +                 RXRAT(   475 )      ! VROCP1ALKP2+NO3=VROCN2OXY2+...
     &                     +                 RXRAT(   480 )      ! VROCP3ALKP2+HO2=VROCN2OXY2
     &                     +                 RXRAT(   481 )      ! VROCP2ALKP2+HO2=VROCN2OXY2
     &                     +                 RXRAT(   482 )      ! VROCP1ALKP2+HO2=VROCN2OXY2
     &                     +    8.8000D-03 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.8800D-...
     &                     +    7.2610D-01 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.7261D+...
     &                     +    1.1560D-01 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.1156D+...
     &                     +    4.0000D-04 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.4000D-...
     &                     +    3.3490D-01 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.3349D+...
     &                     +    1.0400D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.1040D-...
     &                     +    9.0000D-04 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.9000D-...
     &                     +    6.0000D-04 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.6000D-...
      LOSS(  VROCN2OXY2  ) =    7.8590D-01 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.2141D+...

      PROD(  SESQRO2     ) =                 RXRAT(   414 )      ! SESQ+HO=SESQRO2
      LOSS(  SESQRO2     ) =                 RXRAT(   415 )      ! SESQRO2+HO2=VROCP0OXY2
     &                     +                 RXRAT(   416 )      ! SESQRO2+NO3=VROCP3OXY2
     &                     +                 RXRAT(   417 )      ! SESQRO2+NO=0.2470D+...

      PROD(  VROCP0OXY2  ) =                 RXRAT(   415 )      ! SESQRO2+HO2=VROCP0OXY2
     &                     +    2.8000D-01 * RXRAT(   439 )      ! VROCP2ALKP+NO=0.2800D+...
     &                     +                 RXRAT(   452 )      ! VROCP3ALKP+HO2=VROCP0OXY2
     &                     +                 RXRAT(   459 )      ! VROCP3ALKP2=VROCP0OXY2+HO2
     &                     +    1.4000D-01 * RXRAT(   465 )      ! VROCP4ALKP2+NO=0.1400D+...
     &                     +    8.6000D-01 * RXRAT(   466 )      ! VROCP3ALKP2+NO=0.8600D+...
     &                     +                 RXRAT(   473 )      ! VROCP3ALKP2+NO3=VROCP0OXY2+...
     &                     +                 RXRAT(   478 )      ! VROCP5ALKP2+HO2=VROCP0OXY2
     &                     +    9.0480D-01 * RXRAT(   491 )      ! VROCP5AROP+HO2=0.9048D+...
     &                     +    4.5600D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.4560D-...
     &                     +    1.0400D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.1040D-...
     &                     +    4.5400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.4540D-...
     &                     +    7.0000D-04 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.7000D-...
     &                     +    7.6600D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.7660D-...
     &                     +    4.0300D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.4030D-...
     &                     +    1.5500D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.1550D-...
     &                     +    1.5300D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1530D-...
      LOSS(  VROCP0OXY2  ) =    9.9760D-01 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.2400D-...

      PROD(  VROCP1OXY3  ) =    2.4700D-01 * RXRAT(   417 )      ! SESQRO2+NO=0.2470D+...
     &                     +    1.4000D-01 * RXRAT(   464 )      ! VROCP5ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   477 )      ! VROCP6ALKP2+HO2=VROCP1OXY3
     &                     +    9.0480D-01 * RXRAT(   485 )      ! VROCP6AROP+HO2=0.9048D+...
     &                     +    1.8000D-03 * RXRAT(   492 )      ! VROCP5AROP+NO=0.1800D-...
     &                     +    9.0480D-01 * RXRAT(   497 )      ! NAPHP+HO2=0.9048D+...
     &                     +    3.8800D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.3880D-...
     &                     +    3.1400D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3140D-...
     &                     +    5.1000D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.5100D-...
     &                     +    2.3400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.2340D-...
     &                     +    3.9500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.3950D-...
     &                     +    2.7700D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2770D-...
     &                     +    5.1100D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.5110D-...
     &                     +    1.0510D-01 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.1051D+...
     &                     +    9.1600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.9160D-...
     &                     +    1.0380D-01 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1038D+...
     &                     +    5.0300D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.5030D-...
      LOSS(  VROCP1OXY3  ) =    9.9940D-01 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.6000D-...

      PROD(  AGLYJ       ) =                 RXRAT(   418 )      ! GLY=AGLYJ
     &                     +                 RXRAT(   419 )      ! MGLY=AGLYJ
      LOSS(  AGLYJ       ) =   0.0D0

      PROD(  AISONJ      ) =                 RXRAT(   420 )      ! ISON=AISONJ
      LOSS(  AISONJ      ) =   0.0D0

      PROD(  ATRPNJ      ) =                 RXRAT(   421 )      ! TRPN=ATRPNJ
      LOSS(  ATRPNJ      ) =   0.0D0

      PROD(  IEPOXP      ) =                 RXRAT(   425 )      ! IEPOX=IEPOXP
      LOSS(  IEPOXP      ) =                 RXRAT(   426 )      ! IEPOXP=AISO3NOSJ
     &                     +                 RXRAT(   427 )      ! IEPOXP+ASO4J=AISO3OSJ

      PROD(  AISO3NOSJ   ) =                 RXRAT(   426 )      ! IEPOXP=AISO3NOSJ
      LOSS(  AISO3NOSJ   ) =   0.0D0

      PROD(  ASO4J       ) =   0.0D0
      LOSS(  ASO4J       ) =                 RXRAT(   427 )      ! ASO4J+IEPOXP=AISO3OSJ

      PROD(  AISO3OSJ    ) =                 RXRAT(   427 )      ! IEPOXP+ASO4J=AISO3OSJ
      LOSS(  AISO3OSJ    ) =   0.0D0

      PROD(  VROCP6ALK   ) =   0.0D0
      LOSS(  VROCP6ALK   ) =                 RXRAT(   428 )      ! VROCP6ALK+HO=VROCP6ALKP

      PROD(  VROCP6ALKP  ) =                 RXRAT(   428 )      ! VROCP6ALK+HO=VROCP6ALKP
      LOSS(  VROCP6ALKP  ) =                 RXRAT(   435 )      ! VROCP6ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   442 )      ! VROCP6ALKP+NO3=VROCP6ALKP2+NO2
     &                     +                 RXRAT(   449 )      ! VROCP6ALKP+HO2=VROCP3OXY2

      PROD(  VROCP5ALK   ) =   0.0D0
      LOSS(  VROCP5ALK   ) =                 RXRAT(   429 )      ! VROCP5ALK+HO=VROCP5ALKP

      PROD(  VROCP5ALKP  ) =                 RXRAT(   429 )      ! VROCP5ALK+HO=VROCP5ALKP
      LOSS(  VROCP5ALKP  ) =                 RXRAT(   436 )      ! VROCP5ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   443 )      ! VROCP5ALKP+NO3=VROCP5ALKP2+NO2
     &                     +                 RXRAT(   450 )      ! VROCP5ALKP+HO2=VROCP2OXY2

      PROD(  VROCP4ALK   ) =   0.0D0
      LOSS(  VROCP4ALK   ) =                 RXRAT(   430 )      ! VROCP4ALK+HO=VROCP4ALKP

      PROD(  VROCP4ALKP  ) =                 RXRAT(   430 )      ! VROCP4ALK+HO=VROCP4ALKP
      LOSS(  VROCP4ALKP  ) =                 RXRAT(   437 )      ! VROCP4ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   444 )      ! VROCP4ALKP+NO3=VROCP4ALKP2+NO2
     &                     +                 RXRAT(   451 )      ! VROCP4ALKP+HO2=VROCP1OXY1

      PROD(  VROCP3ALK   ) =   0.0D0
      LOSS(  VROCP3ALK   ) =                 RXRAT(   431 )      ! VROCP3ALK+HO=VROCP3ALKP

      PROD(  VROCP3ALKP  ) =                 RXRAT(   431 )      ! VROCP3ALK+HO=VROCP3ALKP
      LOSS(  VROCP3ALKP  ) =                 RXRAT(   438 )      ! VROCP3ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   445 )      ! VROCP3ALKP+NO3=VROCP3ALKP2+NO2
     &                     +                 RXRAT(   452 )      ! VROCP3ALKP+HO2=VROCP0OXY2

      PROD(  VROCP2ALK   ) =   0.0D0
      LOSS(  VROCP2ALK   ) =                 RXRAT(   432 )      ! VROCP2ALK+HO=VROCP2ALKP

      PROD(  VROCP2ALKP  ) =                 RXRAT(   432 )      ! VROCP2ALK+HO=VROCP2ALKP
      LOSS(  VROCP2ALKP  ) =                 RXRAT(   439 )      ! VROCP2ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   446 )      ! VROCP2ALKP+NO3=VROCP2ALKP2+NO2
     &                     +                 RXRAT(   453 )      ! VROCP2ALKP+HO2=VROCN1OXY1

      PROD(  VROCP1ALK   ) =   0.0D0
      LOSS(  VROCP1ALK   ) =                 RXRAT(   433 )      ! VROCP1ALK+HO=VROCP1ALKP

      PROD(  VROCP1ALKP  ) =                 RXRAT(   433 )      ! VROCP1ALK+HO=VROCP1ALKP
      LOSS(  VROCP1ALKP  ) =                 RXRAT(   440 )      ! VROCP1ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   447 )      ! VROCP1ALKP+NO3=VROCP1ALKP2+NO2
     &                     +                 RXRAT(   454 )      ! VROCP1ALKP+HO2=VROCN2OXY2

      PROD(  HC10        ) =   0.0D0
      LOSS(  HC10        ) =                 RXRAT(   434 )      ! HC10+HO=HC10P

      PROD(  VROCP6ALKP2 ) =    7.2000D-01 * RXRAT(   435 )      ! VROCP6ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   442 )      ! VROCP6ALKP+NO3=VROCP6ALKP2+NO2
      LOSS(  VROCP6ALKP2 ) =                 RXRAT(   456 )      ! VROCP6ALKP2=HO2+VROCP3OXY2
     &                     +                 RXRAT(   463 )      ! VROCP6ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   470 )      ! VROCP6ALKP2+NO3=NO2+...
     &                     +                 RXRAT(   477 )      ! VROCP6ALKP2+HO2=VROCP1OXY3

      PROD(  VROCP5ALKP2 ) =    7.2000D-01 * RXRAT(   436 )      ! VROCP5ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   443 )      ! VROCP5ALKP+NO3=VROCP5ALKP2+NO2
      LOSS(  VROCP5ALKP2 ) =                 RXRAT(   457 )      ! VROCP5ALKP2=HO2+VROCP2OXY2
     &                     +                 RXRAT(   464 )      ! VROCP5ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   471 )      ! VROCP5ALKP2+NO3=NO2+...
     &                     +                 RXRAT(   478 )      ! VROCP5ALKP2+HO2=VROCP0OXY2

      PROD(  VROCP4ALKP2 ) =    7.2000D-01 * RXRAT(   437 )      ! VROCP4ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   444 )      ! VROCP4ALKP+NO3=VROCP4ALKP2+NO2
      LOSS(  VROCP4ALKP2 ) =                 RXRAT(   458 )      ! VROCP4ALKP2=HO2+VROCP1OXY1
     &                     +                 RXRAT(   465 )      ! VROCP4ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   472 )      ! VROCP4ALKP2+NO3=NO2+...
     &                     +                 RXRAT(   479 )      ! VROCP4ALKP2+HO2=VROCN1OXY1

      PROD(  VROCP2OXY2  ) =    2.8000D-01 * RXRAT(   437 )      ! VROCP4ALKP+NO=0.2800D+...
     &                     +                 RXRAT(   450 )      ! VROCP5ALKP+HO2=VROCP2OXY2
     &                     +                 RXRAT(   457 )      ! VROCP5ALKP2=VROCP2OXY2+HO2
     &                     +    1.4000D-01 * RXRAT(   463 )      ! VROCP6ALKP2+NO=0.1400D+...
     &                     +    8.6000D-01 * RXRAT(   464 )      ! VROCP5ALKP2+NO=0.8600D+...
     &                     +                 RXRAT(   471 )      ! VROCP5ALKP2+NO3=VROCP2OXY2+...
     &                     +                 RXRAT(   483 )      ! HC10P2+HO2=VROCP2OXY2
     &                     +    1.8000D-03 * RXRAT(   486 )      ! VROCP6AROP+NO=0.1800D-...
     &                     +    1.8000D-03 * RXRAT(   498 )      ! NAPHP+NO=0.1800D-...
     &                     +    4.9100D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.4910D-...
     &                     +    4.5900D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.4590D-...
     &                     +    5.1200D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.5120D-...
     &                     +    6.2400D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.6240D-...
     &                     +    6.6000D-02 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.6600D-...
     &                     +    5.3900D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.5390D-...
     &                     +    2.2700D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.2270D-...
     &                     +    6.5100D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.6510D-...
     &                     +    5.3500D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.5350D-...
     &                     +    9.7200D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.9720D-...
     &                     +    1.6500D-01 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1650D+...
     &                     +    8.7900D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.8790D-...
      LOSS(  VROCP2OXY2  ) =    9.7640D-01 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.2360D-...

      PROD(  VROCP3ALKP2 ) =    7.2000D-01 * RXRAT(   438 )      ! VROCP3ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   445 )      ! VROCP3ALKP+NO3=VROCP3ALKP2+NO2
      LOSS(  VROCP3ALKP2 ) =                 RXRAT(   459 )      ! VROCP3ALKP2=HO2+VROCP0OXY2
     &                     +                 RXRAT(   466 )      ! VROCP3ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   473 )      ! VROCP3ALKP2+NO3=NO2+...
     &                     +                 RXRAT(   480 )      ! VROCP3ALKP2+HO2=VROCN2OXY2

      PROD(  VROCP1OXY1  ) =    2.8000D-01 * RXRAT(   438 )      ! VROCP3ALKP+NO=0.2800D+...
     &                     +                 RXRAT(   451 )      ! VROCP4ALKP+HO2=VROCP1OXY1
     &                     +                 RXRAT(   458 )      ! VROCP4ALKP2=VROCP1OXY1+HO2
     &                     +    8.6000D-01 * RXRAT(   465 )      ! VROCP4ALKP2+NO=0.8600D+...
     &                     +                 RXRAT(   472 )      ! VROCP4ALKP2+NO3=VROCP1OXY1+...
     &                     +    1.9900D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.1990D-...
     &                     +    7.5000D-03 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.7500D-...
     &                     +    3.5200D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.3520D-...
     &                     +    2.1500D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.2150D-...
     &                     +    6.8000D-03 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.6800D-...
     &                     +    1.3000D-03 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.1300D-...
     &                     +    7.3000D-03 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7300D-...
     &                     +    3.1000D-03 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.3100D-...
     &                     +    2.2000D-03 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.2200D-...
      LOSS(  VROCP1OXY1  ) =    9.8820D-01 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.1180D-...

      PROD(  VROCP2ALKP2 ) =    7.2000D-01 * RXRAT(   439 )      ! VROCP2ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   446 )      ! VROCP2ALKP+NO3=VROCP2ALKP2+NO2
      LOSS(  VROCP2ALKP2 ) =                 RXRAT(   460 )      ! VROCP2ALKP2=HO2+VROCN1OXY1
     &                     +                 RXRAT(   467 )      ! VROCP2ALKP2+NO=0.1400D+...
     &                     +                 RXRAT(   474 )      ! VROCP2ALKP2+NO3=NO2+...
     &                     +                 RXRAT(   481 )      ! VROCP2ALKP2+HO2=VROCN2OXY2

      PROD(  VROCP1ALKP2 ) =    7.2000D-01 * RXRAT(   440 )      ! VROCP1ALKP+NO=0.7200D+...
     &                     +                 RXRAT(   447 )      ! VROCP1ALKP+NO3=VROCP1ALKP2+NO2
      LOSS(  VROCP1ALKP2 ) =                 RXRAT(   461 )      ! VROCP1ALKP2=HO2+VROCN2OXY2
     &                     +                 RXRAT(   468 )      ! VROCP1ALKP2+NO=VROCN2OXY2+...
     &                     +                 RXRAT(   475 )      ! VROCP1ALKP2+NO3=NO2+...
     &                     +                 RXRAT(   482 )      ! VROCP1ALKP2+HO2=VROCN2OXY2

      PROD(  VROCN1OXY1  ) =    2.8000D-01 * RXRAT(   440 )      ! VROCP1ALKP+NO=0.2800D+...
     &                     +                 RXRAT(   453 )      ! VROCP2ALKP+HO2=VROCN1OXY1
     &                     +                 RXRAT(   460 )      ! VROCP2ALKP2=VROCN1OXY1+HO2
     &                     +    1.4000D-01 * RXRAT(   466 )      ! VROCP3ALKP2+NO=0.1400D+...
     &                     +    8.6000D-01 * RXRAT(   467 )      ! VROCP2ALKP2+NO=0.8600D+...
     &                     +                 RXRAT(   474 )      ! VROCP2ALKP2+NO3=VROCN1OXY1+...
     &                     +                 RXRAT(   479 )      ! VROCP4ALKP2+HO2=VROCN1OXY1
     &                     +    9.6000D-03 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.9600D-...
     &                     +    4.6000D-03 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4600D-...
     &                     +    7.5800D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.7580D-...
     &                     +    1.3000D-03 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.1300D-...
      LOSS(  VROCN1OXY1  ) =    9.9300D-01 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.7000D-...

      PROD(  HC10P2      ) =    7.4000D-01 * RXRAT(   441 )      ! HC10P+NO=0.7400D+00*HC10P2+...
     &                     +                 RXRAT(   448 )      ! HC10P+NO3=HC10P2+NO2
      LOSS(  HC10P2      ) =                 RXRAT(   462 )      ! HC10P2=HO2+VROCP4OXY2
     &                     +                 RXRAT(   469 )      ! HC10P2+NO=0.1200D+00*ONIT+...
     &                     +                 RXRAT(   476 )      ! HC10P2+NO3=NO2+KET+HO2
     &                     +                 RXRAT(   483 )      ! HC10P2+HO2=VROCP2OXY2

      PROD(  VROCP6ARO   ) =   0.0D0
      LOSS(  VROCP6ARO   ) =                 RXRAT(   484 )      ! VROCP6ARO+HO=0.8400D+...

      PROD(  VROCP6AROP  ) =    8.4000D-01 * RXRAT(   484 )      ! VROCP6ARO+HO=0.8400D+...
      LOSS(  VROCP6AROP  ) =                 RXRAT(   485 )      ! VROCP6AROP+HO2=0.5950D-...
     &                     +                 RXRAT(   486 )      ! VROCP6AROP+NO=0.1000D-...
     &                     +                 RXRAT(   487 )      ! VROCP6AROP+NO3=NO2+0.9405D+...
     &                     +                 RXRAT(   488 )      ! VROCP6AROP+MO2=0.6800D+...
     &                     +                 RXRAT(   489 )      ! VROCP6AROP+ACO3=0.7000D+...

      PROD(  VROCN2OXY4  ) =    3.5700D-02 * RXRAT(   485 )      ! VROCP6AROP+HO2=0.3570D-...
     &                     +    3.5700D-02 * RXRAT(   491 )      ! VROCP5AROP+HO2=0.3570D-...
     &                     +    1.0000D-04 * RXRAT(   492 )      ! VROCP5AROP+NO=0.1000D-...
     &                     +    5.6380D-01 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.5638D+...
     &                     +    7.1000D-03 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.7100D-...
     &                     +    4.0250D-01 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.4025D+...
     &                     +    1.1900D-01 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.1190D+...
     &                     +    1.1650D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1165D+...
     &                     +    4.5790D-01 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4579D+...
     &                     +    1.9240D-01 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.1924D+...
     &                     +    1.3400D-01 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.1340D+...
     &                     +    1.7260D-01 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.1726D+...
     &                     +    7.6500D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.7650D-...
     &                     +    1.6700D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.1670D-...
     &                     +    1.0300D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1030D-...
     &                     +    8.0000D-04 * RXRAT(   517 )      ! OP3+HO=0.8000D-...
      LOSS(  VROCN2OXY4  ) =    8.0230D-01 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.1977D+...

      PROD(  VROCN1OXY3  ) =    1.0000D-04 * RXRAT(   486 )      ! VROCP6AROP+NO=0.1000D-...
     &                     +    1.5200D-02 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.1520D-...
     &                     +    2.9800D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.2980D-...
     &                     +    3.0500D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.3050D-...
     &                     +    1.8300D-02 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.1830D-...
     &                     +    6.5700D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.6570D-...
     &                     +    4.5200D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.4520D-...
     &                     +    1.1930D-01 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.1193D+...
     &                     +    1.1180D-01 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.1118D+...
     &                     +    4.8900D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.4890D-...
     &                     +    2.4600D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.2460D-...
     &                     +    1.4600D-02 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1460D-...
     &                     +    4.9000D-03 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.4900D-...
      LOSS(  VROCN1OXY3  ) =    9.9240D-01 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.7600D-...

      PROD(  VROCP5ARO   ) =   0.0D0
      LOSS(  VROCP5ARO   ) =                 RXRAT(   490 )      ! VROCP5ARO+HO=0.8400D+...

      PROD(  VROCP5AROP  ) =    8.4000D-01 * RXRAT(   490 )      ! VROCP5ARO+HO=0.8400D+...
      LOSS(  VROCP5AROP  ) =                 RXRAT(   491 )      ! VROCP5AROP+HO2=0.5950D-...
     &                     +                 RXRAT(   492 )      ! VROCP5AROP+NO=0.1000D-...
     &                     +                 RXRAT(   493 )      ! VROCP5AROP+NO3=NO2+0.9405D+...
     &                     +                 RXRAT(   494 )      ! VROCP5AROP+MO2=0.6800D+...
     &                     +                 RXRAT(   495 )      ! VROCP5AROP+ACO3=0.7000D+...

      PROD(  NAPH        ) =   0.0D0
      LOSS(  NAPH        ) =                 RXRAT(   496 )      ! NAPH+HO=0.8400D+00*NAPHP+...

      PROD(  NAPHP       ) =    8.4000D-01 * RXRAT(   496 )      ! NAPH+HO=0.8400D+00*NAPHP+...
      LOSS(  NAPHP       ) =                 RXRAT(   497 )      ! NAPHP+HO2=0.5950D-...
     &                     +                 RXRAT(   498 )      ! NAPHP+NO=0.5950D-...
     &                     +                 RXRAT(   499 )      ! NAPHP+NO3=NO2+0.9405D+...
     &                     +                 RXRAT(   500 )      ! NAPHP+MO2=0.6800D+00*HCHO+...
     &                     +                 RXRAT(   501 )      ! NAPHP+ACO3=0.7000D+00*MO2+...

      PROD(  VROCN2OXY8  ) =    3.5700D-02 * RXRAT(   497 )      ! NAPHP+HO2=0.3570D-...
     &                     +    1.0000D-04 * RXRAT(   498 )      ! NAPHP+NO=0.1000D-...
     &                     +    4.6400D-01 * RXRAT(   503 )      ! VROCN2OXY4+HO=0.4640D+...
     &                     +    1.0410D-01 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.1041D+...
     &                     +    2.0360D-01 * RXRAT(   505 )      ! VROCN1OXY6+HO=0.2036D+...
     &                     +    2.7920D-01 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.2792D+...
     &                     +    7.4000D-03 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.7400D-...
     &                     +    2.8220D-01 * RXRAT(   508 )      ! VROCP0OXY4+HO=0.2822D+...
     &                     +    6.5900D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.6590D-...
     &                     +    1.7780D-01 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.1778D+...
     &                     +    2.3000D-03 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.2300D-...
     &                     +    4.4500D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.4450D-...
     &                     +    3.1700D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.3170D-...
     &                     +    1.1700D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.1170D-...
     &                     +    1.1880D-01 * RXRAT(   517 )      ! OP3+HO=0.1188D+...
      LOSS(  VROCN2OXY8  ) =    9.1460D-01 * RXRAT(   502 )      ! VROCN2OXY8+HO=0.8540D-...

      PROD(  VROCP5OXY1  ) =    3.2500D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3250D-...
     &                     +    1.3800D-02 * RXRAT(   506 )      ! VROCN1OXY3+HO=0.1380D-...
     &                     +    3.4700D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.3470D-...
     &                     +    3.7100D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.3710D-...
     &                     +    1.7400D-02 * RXRAT(   510 )      ! VROCP1OXY3+HO=0.1740D-...
     &                     +    4.2300D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.4230D-...
     &                     +    5.2300D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.5230D-...
     &                     +    5.8200D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.5820D-...
     &                     +    4.7900D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.4790D-...
     &                     +    4.3200D-02 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.4320D-...
      LOSS(  VROCP5OXY1  ) =    9.9380D-01 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.6200D-...

      PROD(  VROCP6OXY1  ) =    3.6900D-02 * RXRAT(   504 )      ! VROCN2OXY2+HO=0.3690D-...
     &                     +    7.4800D-02 * RXRAT(   507 )      ! VROCN1OXY1+HO=0.7480D-...
     &                     +    4.2100D-02 * RXRAT(   509 )      ! VROCP0OXY2+HO=0.4210D-...
     &                     +    9.1100D-02 * RXRAT(   511 )      ! VROCP1OXY1+HO=0.9110D-...
     &                     +    5.9500D-02 * RXRAT(   512 )      ! VROCP2OXY2+HO=0.5950D-...
     &                     +    6.6100D-02 * RXRAT(   513 )      ! VROCP3OXY2+HO=0.6610D-...
     &                     +    7.4500D-02 * RXRAT(   514 )      ! VROCP4OXY2+HO=0.7450D-...
     &                     +    1.3980D-01 * RXRAT(   515 )      ! VROCP5OXY1+HO=0.1398D+...
      LOSS(  VROCP6OXY1  ) =    9.0430D-01 * RXRAT(   516 )      ! VROCP6OXY1+HO=0.9570D-...

      RETURN

      END
